/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DllLoadTimeSP.c
 *
 *   Path    $(TESTSUITEROOT)/Analysis/DynLink/
 *
 *   Desc:
 *           Measures the load time for split phase DLL
 *           with different memory configurations.
 *
 *  Usage:
 *        DLLLOADTIMESP [-v] [-l "log path/filename"]
 *                      [-x "valid executable"] ..
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: DSP Image to be used
 *        -t: Number of Threads to be created
 *        -n: Number of Transfers for each Iteration
 *        -s: Buffer Size
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 *
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUN 23, 2003    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>
#include "dynLink.h"

#define DEFNAMESIZE      100
#define DEFBUFSIZE        64
#define DEFNUMBUFS         1
#define DEFNUMTXFRS    10000
#define DEFNUMTHREADS     10
#define NUMOFCOMPARGS      3	/* Compulsory args: t,n,s */
#define PASS               0
#define FAIL               1

#define SHORT short

#define DSPEXEC          "inout_nortdx_int_1510.dof55L"
#define DLLNAME          "inout5nsp.dll55L"
#define CREATEPHDLLNAME  "inout5nCreatedyn.dll55L"
#define EXECPHDLLNAME    "inout5nExecutedyn.dll55L"
#define DELETEPHDLLNAME  "inout5nDeletedyn.dll55L"


/* Function to exit gracefully */
static INT graceful_exit()
{
	/*printf("FATAL ERROR: Exiting the Test Case !\n");
	exit(0);*/
}

/* Thread function for Dynamic Linking test case */
/*
 * Note: Address of args is being passed from the main thread,		*
 * dont alter the contents as this will be used/needed by all the	*
 * threads created. Copy the contents to local variables and use	*
 */
static INT threadFxn(dynLinkArgs * targs)
{

    struct DSP_STRMATTR			attrs;
	BYTE					**arrBuf;
    struct DSP_NODEATTRIN			nodeAttrin;
    DSP_HNODE				hNode = NULL;
	CHAR					*pszUuid;
	struct DSP_UUID				gidNodeID;
    struct DSP_CBDATA				*cbData;
	DSP_STATUS				status;
	DSP_STATUS				exitStatus = DSP_SOK;

	/* Processor related information */
	DSP_HPROCESSOR			hProcessor;

	/* Test case specific information */
	INT						iNumTransfers;
	INT						iBuffSize;

	/* Logging related information */
	FILE					*pHandle;
	BOOL					bUseLog=FALSE;
	BOOL					bVerbose=FALSE;
	 BYTE                            argsBuf[32 + sizeof(ULONG)];
	 BYTE				**arrBuf1;
	 CHAR				* dllName;

	DWORD			dwStartTime;
	DWORD			dwEndTime;
	DWORD         	dwTotalElapsedTime;

	/* Copy the contents of passed args to local vars */
	/* This strategy can be changed if the variables are just
	   used/read and not modified. */
	hProcessor = targs->hProcessor;
	iNumTransfers = targs->iNumTransfers;
	iBuffSize = targs->iBuffSize;
	pHandle = targs->pHandle;
	bUseLog = targs->bUseLog;
	bVerbose = targs->bVerbose;
	pszUuid = targs->uuidPtr;
	dllName = targs->dllName;


    /* Initialising attributes */
    attrs.uSegid = 0;
    attrs.uBufsize = iBuffSize;
    attrs.lMode = targs->transferMode;
    attrs.uNumBufs = DEFNUMBUFS;
    attrs.uAlignment = 0;
    attrs.uTimeout = 5000;

	if (targs->transferMode  == STRMMODE_PROCCOPY)
    {
		attrs.uSegid = 0;
    }
    else
    {
		attrs.uSegid = DSP_SHMSEG0;    /* Use shared memory segment 0 */
    }

    nodeAttrin.iPriority = 5 ;
    nodeAttrin.uTimeout = 10000 ;

#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif

   cbData = (struct DSP_CBDATA *)argsBuf ;

   cbData->cbData = 32;
	arrBuf = OsSpecificMemAlloc(sizeof(BYTE *) * DEFNUMBUFS);
	arrBuf1 = OsSpecificMemAlloc(sizeof(BYTE *) * DEFNUMBUFS);


	// pszUuid is passed from calling function
	/*printf("UUID is = %s\n",pszUuid);
	printf("DLL  is = %s\n",dllName);*/
    UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

    /* Register the node information with DCD */
    /*status = DSPManager_RegisterObject(&gidNodeID, DSP_DCDNODETYPE, dllName);
    if (!DSP_SUCCEEDED(status))
	{
        TST_verbose("FAIL\n");
        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL status= 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPManager_RegisterObject().\n");
        }
        return FAIL;
    }*/

    /* Allocate Node */
    status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData, &nodeAttrin,
                   &hNode);

    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Allocate().\n");
        }
        return FAIL;
    }

    /*printf("After Allocate\n");*/

    status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect()\n");
        }
        return FAIL;
    }

    status = DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect()\n");
        }
        return FAIL;
    }

    /*printf("After Connect\n");*/

    OsSpecificTimerStart(&dwStartTime);

    /* Create Node on DSP */
    status = DSPNode_Create(hNode);

    OsSpecificTimerStop(&dwEndTime);
    OsSpecificTimerDifference(&dwEndTime, &dwStartTime, &dwTotalElapsedTime);
    //dwAvgTime = dwTotalElapsedTime / 100;

    //printf("SSSSSSSSSSSSSSSS StartTime = %d\n", dwStartTime);
    //printf("EEEEEEEEEEEEEEEE EndTime = %d\n", dwEndTime);
    /*printf("CCCCCCCCCC ElapsedTime = %d mSec\n", dwTotalElapsedTime); */
    //printf("Average ElapsedTime = %d mSec\n", dwAvgTime);


    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Create()\n");
        }
        return FAIL;
    }

    OsSpecificTimerStart(&dwStartTime);
    status = DSPNode_Run(hNode);
    OsSpecificTimerStop(&dwEndTime);
    OsSpecificTimerDifference(&dwEndTime, &dwStartTime, &dwTotalElapsedTime);
    /*printf("RRRRRRRRRR ElapsedTime = %d mSec\n", dwTotalElapsedTime); */

    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Run()\n");
        }
        return FAIL;
    }

    /*printf("After Run\n");*/

    status = DSPNode_Terminate(hNode, &exitStatus);
	if (!DSP_SUCCEEDED(status))
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog)
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPNode_Terminate()\n");
		}
		/*printf("Error in DSPNode_Terminate\n");*/
		return FAIL;
	}

    OsSpecificTimerStart(&dwStartTime);
    status = DSPNode_Delete(hNode);
    OsSpecificTimerStop(&dwEndTime);
    OsSpecificTimerDifference(&dwEndTime, &dwStartTime, &dwTotalElapsedTime);
    /*printf("DDDDDDDDDD ElapsedTime = %d mSec\n", dwTotalElapsedTime); */

	if (!DSP_SUCCEEDED(status))
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog)
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPNode_Delete()\n");
		}
		/*printf("Error in DSPNode_Delete()\n");*/
		return FAIL;
	}

	OsSpecificMemFree(arrBuf) ;
	OsSpecificMemFree(arrBuf1) ;
    /* Return from this Thread */
    if (DSP_SUCCEEDED(status))
    {
        return PASS;
    }
    else
    {
        return FAIL;
    }
}



/*
 *  ======== main ========
 *
 */
INT dllLoadTimeSP_ANALYSIS(INT argc, CHAR * argv[])
{
    BOOL           			bUseLog=FALSE;

    BOOL           			bVerbose=FALSE;
    FILE           			*pHandle = NULL;

	INT            			iNumThreads = DEFNUMTHREADS;
    INT            			iNumTransfers = DEFNUMTXFRS;
    INT            			iBuffSize = DEFBUFSIZE;

	CHAR           			szLogName[DEFNAMESIZE];
	INT            			iMandArgCount = NUMOFCOMPARGS;
	DSP_STATUS     			status;
	dynLinkArgs 			thrdArgs;
    struct DSP_PROCESSORATTRIN		*pAttrIn;
	DSP_HPROCESSOR			hProcessor;
    CHAR					*aArgv[DEFNAMESIZE] = { DSPEXEC, "" };
	INT						iArgc = 1;
	INT						opt;
	INT						index;
    INT                 	retVal = PASS;
	INT 					transferMode[] =
							{
								STRMMODE_PROCCOPY,
								STRMMODE_PROCCOPY,
								STRMMODE_PROCCOPY,
								STRMMODE_PROCCOPY,
								STRMMODE_RDMA
							};
	CHAR					*uuidArr[] =
							{
								"7268FD29_2E18_45E6_86D7_83F243D91201", // ainout
								"27C58E71_C057_4E07_8126_5FF867CE903B", // binout
								"3C45DA77_7882_44BD_BC7C_319597B3EEC0", // cinout
								"A36AA3DD_CA21_475F_BF64_8C9860C418AA", // dinout
								"69040FCA_92C0_4E63_B209_DF7F42D6D204"  // einout
							};
	CHAR        *aDllName[DEFNAMESIZE] = { DLLNAME, "arg3" };
	struct DSP_UUID				gidNodeID;
	CHAR		*creatPhDll = CREATEPHDLLNAME;
	CHAR		*execPhDll = EXECPHDLLNAME;
	CHAR		*delPhDll = DELETEPHDLLNAME;

	struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    pindex=0;
        INT                     procId;



	/* Check Number of Arguments */
	/* Process arguments and Store the values */
	/* Load the Image on DSP */
	/* Create requested number of Threads and pass the
	   input params to these Threads */
	/* In each of the Thread do the following: */
		/* Allocate Buffers for Transfer of Data */
		/* Set Properties, dynamically Allocate,Create the Node on DSP */
		/* Open the Streams */
		/* Run the task */
		/* Do the Data Transfer */
	/* End !! */


	/* Process arguments and Store the values */
    optind = 0;
	opt = getopt_test(argc, argv, "+vl:x:t:n:s:d:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DynamicLink_BEHAVIOUR: Verbose Mode On\n");
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "dynLink test:");
            TST_verbose("Log opened as: %s\n", szLogName);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            break;
		case 't':
			/* Number of Threads to be created */
			iNumThreads = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Number of Threads to be Created = %d\n",iNumThreads);
            TSTLOG_appendLog(pHandle,"Number of Threads to be Created = %d\n",iNumThreads);
			break;
		case 'n':
			/* Number of Transfers */
			iNumTransfers = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Number of transfers set to %d\n",iNumTransfers);
            TSTLOG_appendLog(pHandle, "Number of transfer set to %d\n",iNumTransfers);
			break;
		case 's':
			/* Buffer Size */
			iBuffSize = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Buffer Size set to %d\n",iBuffSize);
            TSTLOG_appendLog(pHandle, "Buffer Size set to %d\n",iBuffSize);
			break;
		case 'd':
        	/* use given dll */
        	aDllName[0] = (char *) optarg;
            break;
        default:
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt_test(argc, argv, "+vl:x:t:n:s:d:");
    }

	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		// Error
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing\n");
	}

    TST_verbose("DSP Executable: %s\n", aArgv[0]);


    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
	if (pAttrIn == NULL)
	{
		// Error
		TST_verbose("Malloc for pAttrIn FAILED !!\n");
		graceful_exit();
	}
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Attach to Processor */
	while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(pindex,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
        {
                if (dspInfo.uProcessorType == DSPTYPE_55)
                {
                        TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                        procId = pindex;
                        status = DSP_SOK;
                        break;
                }
                pindex++;
        }
    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, pAttrIn,&hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
	    graceful_exit();
        }
    }
        else
	{
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
            graceful_exit();
	}



    /* Load default program */
    status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, NULL);
    if (!DSP_SUCCEEDED(status))
	{
		TSTLOG_appendLog(pHandle, "Status Val: %x\n",status);
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Load()\n");
        }
		graceful_exit();
    }

    status = DSPProcessor_Start(hProcessor);
    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Start()\n");
        }
        graceful_exit();
    }

    /*printf("After Proc Start\n");*/

    /* Register the node information with DCD */
    for (index = 0; index < iNumThreads; index ++)
    {
	    /*printf("checkpoint 1\n");*/
	    UUID_UuidFromString((CHAR *)uuidArr[index], &gidNodeID);
	    status = DSPManager_RegisterObject(&gidNodeID, DSP_DCDNODETYPE, aDllName[0]);
	    if (!DSP_SUCCEEDED(status))
		{
	        TST_verbose("FAIL\n");
	        if (bUseLog)
			{
	            TSTLOG_appendLog(pHandle, "FAIL status= 0x%x\n",status);
	            TSTLOG_appendLog(pHandle,"Error in DSPManager_RegisterObject().\n");
	        }
	        return FAIL;
	    }

	    /*printf("checkpoint 2\n");*/
	    status = DSPManager_RegisterObject(&gidNodeID,DSP_DCDCREATELIBTYPE,creatPhDll);
	    /*printf("cehckpoint 3\n");*/
	    status = DSPManager_RegisterObject(&gidNodeID,DSP_DCDEXECUTELIBTYPE,execPhDll);
	    /*printf("checkpoint 4\n");*/
	    status = DSPManager_RegisterObject(&gidNodeID,DSP_DCDDELETELIBTYPE,delPhDll);
	    /*printf("checkpoint 5\n");*/

	    if (!DSP_SUCCEEDED(status))
		{
	        TST_verbose("FAIL\n");
	        if (bUseLog)
			{
	            TSTLOG_appendLog(pHandle, "FAIL status= 0x%x\n",status);
	            TSTLOG_appendLog(pHandle,"Error in DSPManager_RegisterObject().\n");
	        }
	        return FAIL;
	    }
    }

    /*printf("After reg objects\n");*/

	/* Fill the thrdArgs which are to be passed on to the Thread */
	thrdArgs.hProcessor = hProcessor;
	thrdArgs.iNumTransfers = iNumTransfers;
	thrdArgs.iBuffSize = iBuffSize;
	thrdArgs.pHandle = pHandle;
	thrdArgs.bUseLog = bUseLog;
	thrdArgs.bVerbose = bVerbose;
	thrdArgs.dllName = aDllName[0];

	/* Create requested number of Threads and pass the
	   input params to these Threads */

	for (index = 0; index < iNumThreads; index ++)
	{
		thrdArgs.ithrdNum = index ;
		thrdArgs.uuidPtr = uuidArr[index];
		thrdArgs.transferMode = transferMode[index] ;
		/*printf("Trying to call function %d\n",index);*/
		status = threadFxn(&thrdArgs);
	}


    /* UnRegister the node information with DCD */
    for (index = 0; index < iNumThreads; index ++)
    {
	    UUID_UuidFromString((CHAR *)uuidArr[index], &gidNodeID);
	    status = DSPManager_UnregisterObject(&gidNodeID, DSP_DCDNODETYPE);
	    if (!DSP_SUCCEEDED(status))
		{
	        TST_verbose("FAIL\n");
	        if (bUseLog)
			{
	            TSTLOG_appendLog(pHandle, "FAIL status= 0x%x\n",status);
	            TSTLOG_appendLog(pHandle,"Error in DSPManager_UnregisterObject().\n");
	        }
	        return FAIL;
	    }
    }


    DSPProcessor_Detach(hProcessor);

    OsSpecificMemFree(pAttrIn);

    if (retVal == PASS)
	{
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

