/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
*   File    DSPLoadImage.c
*
*   Path    $(TESTSUITEROOT)\Analysis\DSPLoadImage
*
*   Desc:
*      Host initializes and sends a data buffer on the output stream. 
*	   DSP sends the same buffer on the DSP's output stream.Demonstrates
*      data streaming using n-way buffering to/from a DSP program, and 
*      end-of-stream handling.
*
*   Rev     1.0.0
*
*   This computer program is copyright to Texas Instruments Incorporated.
*   The program may not be used without the written permission
*   of Texas Instruments Incorporated. or against the terms and conditions
*   stipulated in the agreement under which this program has been
*   supplied.
*
*   (c) Texas Instruments Incorporated 2003
*
*   ============================================================================
*   Revision History
*
*   SEP 15, 2002  REF=ORG            Raghunandan NagarajaRao.
*      Original Version.
*   ============================================================================
*/
#include <stdio.h> 
#include <dbapi.h>
#include <rms_sh.h>
#include <getopt_test.h>
#include <string.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>   
#include <DSPProcessor_OEM.h>


#define	DEFNAMESIZE         80
#define DSPEXEC             "inout_nortdx_int_1510.dof55L"
#define DEFAULTDSPUNIT      0           /* Default unit number for the board. */

static CHAR			        szLogName[DEFNAMESIZE];

static CONST  CHAR	        *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1"};
static BOOL			        fLoadDsp = FALSE, fOpenLog = FALSE;
static CONST  CHAR	        *aEnvp[DEFNAMESIZE] = {NULL};
static FILE			        *pHandle;
static INT                  NumIterations = 1;

/* Stream copy task context data structure. */
typedef struct {
    DSP_HPROCESSOR          hProcessor;     /* Handle to processor. */
    struct DSP_UUID                hNodeID;        /* Handle to node ID GUID. */
    DSP_HNODE               hNode;          /* Handle to node. */
    DSP_HSTREAM             hOutStream;     /* Handle to node input stream. */
    DSP_HSTREAM             hInStream;      /* Handle to node output stream. */
    BYTE                    **ppInBufs;     /* Input stream buffers. */
    BYTE                    **ppOutBufs;    /* Output stream buffers. */
    struct DSP_NOTIFICATION        hNotification;  /* DSP notification object.*/
    INT                     nStrmMode;      /* stream mode */
} STRMCOPY_TASK; 


static DSP_STATUS InitializeProcessor(STRMCOPY_TASK *copyTask);


/* 
 *  ======== dslLoadImage ========
 */
int dslLoadImage(int argc, char ** argv)
{

   	STRMCOPY_TASK           copyTask;
	DSP_STATUS              status = DSP_SOK;
    	fLoadDsp = FALSE;
    	INT                     opt = 0;
	copyTask.hProcessor = NULL;

	optind = 0;
	  while ((opt = getopt_test(argc, (char * const)argv, "+vl:x:n:")) != EOF) {
        switch (opt) {
         case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;

	      case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
			pHandle = TSTLOG_openLog(szLogName);
			fOpenLog = TRUE;
			break;

          case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
			fLoadDsp = TRUE;
			/* wait for user */
			break;

          case 'n':
            /* Number of times to load the image */
            NumIterations = CSL_Atoi(optarg);
            break;

		  default:
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            break;
        }
    }

    /* Attach to processor */
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId=0;
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, NULL, &copyTask.hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    }
    
	if (DSP_SUCCEEDED(status)) 
	{
        /* Load user specified DSP Executable and start the processor. */
		if(fLoadDsp)
			status = InitializeProcessor(&copyTask);
    }


    if (copyTask.hProcessor) 
	{
        DSPProcessor_Detach(copyTask.hProcessor);
        copyTask.hProcessor = NULL;
    }

    return (DSP_SUCCEEDED(status) ? 0 : -1);
}


/*
 *  ======== InitializeProcessor ========
 *  Perform node related initialization.
 */
static DSP_STATUS InitializeProcessor(STRMCOPY_TASK *copyTask)
{

	  DSP_STATUS    status = DSP_SOK;
	  INT			iArgc = 1;
	  DWORD			dwStartTime;  
	  DWORD			dwAvgTime = 0;
	  DWORD			dwEndTime;
      DWORD         dwTotalElapsedTime;
      INT           i = 0;  
	  
      OsSpecificTimerStart(&dwStartTime);
      for( i = 0; i < NumIterations; i++)
      {
          status = DSPProcessor_Load(copyTask->hProcessor, iArgc, aArgv, aEnvp); 
	  if(DSP_FAILED(status))
		{
		 TSTLOG_appendLog(pHandle, "DSPProcessor_Load is failed in iteration %d and status is 0x%x\n",i,status);
		 break;
		}
      }
      OsSpecificTimerStop(&dwEndTime);
      
      if (DSP_SUCCEEDED(status)) 
	  {
            OsSpecificTimerDifference(&dwEndTime, &dwStartTime, &dwTotalElapsedTime);
            dwAvgTime = dwTotalElapsedTime / NumIterations;
            if(fOpenLog)
            {
		        TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");
		        TSTLOG_appendLog(pHandle, "LOG: *********DSP LOAD IMAGE STATISTICS**********\n");
		        TSTLOG_appendLog(pHandle, "LOG: StartTime = %d\n", dwStartTime); 
		        TSTLOG_appendLog(pHandle, "LOG: EndTime = %d\n", dwEndTime); 
		        TSTLOG_appendLog(pHandle, "LOG: Average ElapsedTime = %d mSec\n", dwAvgTime); 
		        TSTLOG_appendLog(pHandle, "LOG: **********END STATISTICS***********\n");
		        TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");
            }
	  }
	  else
	  {
   			/* Report Error */
			if(fOpenLog)
				TSTLOG_appendLog(pHandle,"Error in DSPProcessor_Load. STATUS = 0x%x\n", status);	
            /* Start Processor */
			status = DSPProcessor_Start(copyTask->hProcessor);
			if (DSP_FAILED(status)) 
			{	
                if(fOpenLog)
			        TSTLOG_appendLog(pHandle, "Error in DSPProcessor Start. status = 0x%x\n", status);
	  		}
	  }
      return (status);

}

