/*
 *  ======== wdmmcopy.c ========
 *  Description:
 *      wdmmcopy demonstrates usage of the Dynamic Memory Mapping APIs
 *      by creating two MPU buffers, mapping them to the DSP virtual
 *      address space, and copying data on the DSP from one buffer to
 *      another.  Bridge messaging is used to synchronize with the DSP.
 *
 *  Usage:
 *      iva-dmmcopy <input-filename> <output-filename>
 *
 *  Example:
 *      iva-dmmcopy "release\input.txt" "release\output.txt"
 *
 *  Notes:
 *      The GPP reads data from an input file and copies the data into
 *      an MPU buffer that has been mapped to the DSP virtual address
 *      space using the DMM APIs.  The GPP then notifies the DSP that
 *      the file read has completed and includes the number of words
 *      that were read.  Using this information, the DSP copies the data
 *      from the MPU mapped buffer to another buffer, which is also an
 *      MPU buffer that has been mapped to the DSP virtual address space.
 *      The DSP then notifies the GPP that the copy operation has completed,
 *      which prompts the GPP to take the data stored in the buffer and
 *      write it to the output file.
 *
 *! Revision History:
 *! ================
 *! 01-Sep-2004 RG : Modified for the Testsuite
 *! 19-Apr-2004 sb : Aligned DMM definitions with Symbian
 *! 19-Mar-2004 map: Created based on wstrmcopy.c
 */



#include <stdio.h>
#include <dbapi.h>
#include <rms_sh.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <string.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>
#include <msgtxfr.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>

/* Application sepcific attributes */

#define DEFAULTDSPUNIT      0       /* Default unit number for the board. */
#define DEFAULTIVAUNIT 1	   /* Processor ID for IVA */
#define DEFAULTIVAEXEC "./iva.dof"
#define ARGSIZE             32

/* Messaging DSP commands */
#define DMM_SETUPBUFFERS    4
#define DMM_WRITEREADY      5

#define IVA_VAOFFSET	0x80000000
#define PHYS_ADDR1  0x82000000
#define PHYS_ADDR2  0x82500000


/* IVA_DMMCOPY_TI_uuid = 860B1B83_DDA2_4AFF_9DE3_F3652CDEE3E1 */
static const struct DSP_UUID IVA_DMMCOPY_TI_uuid = {
	0x860b1b83, 0xdda2, 0x4aff, 0x9d, 0xe3, {
	0xf3, 0x65, 0x2c, 0xde, 0xe3, 0xe1
    }
};



/* Forward declarations: */
static DSP_STATUS CheckFiles(char* inFile,char* outFile);

/* Initialization and cleanup routines. */
static DSP_STATUS InitializeIVAProcessor(IVAMSG_TASK *ivaDmmTask,char *IvaImage);
static DSP_STATUS InitializeIVANode(IVAMSG_TASK *ivaDmmTask);
static DSP_STATUS StopIVAProcessor(IVAMSG_TASK *ivaDmmTask);
static DSP_STATUS CleanupIVAProcessor(IVAMSG_TASK *ivaDmmTask);
static DSP_STATUS CleanupIVANode(IVAMSG_TASK *ivaDmmTask);
/*static DSP_STATUS RunIVATask(IVAMSG_TASK *ivaDmmTask, FILE *inFile,FILE *outFile,INT BufferSize);*/
static DSP_STATUS RunIVATask(IVAMSG_TASK *ivaDmmTask, FILE *inFile,FILE *outFile,INT BufferSize,MAPModel_t *mapModel);

extern DBAPI DSPProcessor_UnMapMemory(DSP_HPROCESSOR hProcessor,PVOID pMapAddr);
extern DBAPI DSPProcessor_Map(DSP_HPROCESSOR hProcessor,PVOID pMpuAddr,ULONG ulSize,PVOID pReqAddr,PVOID *ppMapAddr,ULONG ulMapAttr);

extern DBAPI DSPProcessor_UnMap(DSP_HPROCESSOR hProcessor,PVOID pMapAddr);
extern DBAPI DSPProcessor_FlushMemory (DSP_HPROCESSOR hProcessor,PVOID pMpuAddr,ULONG ulSize,ULONG ulFlags);


/*
 *  ======== IvaDmmCopy ========
 */
INT IvaPwrDmmCopyNeg(INT argc, char ** argv)
{
        
	IVAMSG_TASK     ivaDmmTask;
      DSP_STATUS      status = DSP_SOK;
      INT             opt = 0;
      fLoadDsp = FALSE;
      ivaDmmTask.hProcessorIva = NULL;
      ivaDmmTask.hNodeIva = NULL;
      ivaDmmTask.numMsg = NULL;
      ivaDmmTask.numMsg = NULL;
	char 		*buff_size;
	char 		*IvaImage;
	INT 		BufferSize=0;
	char		*inputfile;
	char		*outputfile;
      FILE 		*inFile=NULL;
      FILE 		*outFile=NULL;
	MAPModel_t      *mapModel;
	char		*mapType;


        optind = 0;
          while ((opt = getopt_test(argc, (char * const)argv, "+vl:x:n:s:f:o:b:t:ZPR:")) != EOF) 
	   {
	        switch (opt) 
	       {
        		case 'v':
          		  /* verbose */
		        TST_setVerbose(TRUE);
		        break;

		        case 'l':
            /* use log and store with name location */
            		strncpy(szLogName, optarg, sizeof(szLogName));
                        pHandle = TSTLOG_openLog(szLogName);
                        fOpenLog = TRUE;
		        TSTLOG_appendLog(pHandle, "Log File Set to %s\n:", szLogName);
                        break;

        		case 'x':
            /* use given executable */
	            	IvaImage = (char *) optarg;
                        fLoadDsp = TRUE;
                        TSTLOG_appendLog(pHandle, "New Executable %s\n:", optarg);
			break;

                	case 'n':
            /* set Number of transfers */
                        ivaDmmTask.numMsg = CSL_Atoi(optarg);
                        if(fOpenLog)
                               TSTLOG_appendLog(pHandle, "Number of Data Transfers =  %i\n", ivaDmmTask.numMsg);
                        break;

                /* Input File Name */
	                case 'f':
                        inputfile=optarg;
                        break;
		/* Output file name*/
        	        case 'o':
                        outputfile=optarg;
                        break;
			case 'b':
			buff_size=optarg;
			break;

			case 't':
	   		mapType=optarg;
	    		break;

        		default:
            		TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            		break;

		}

          }
		DspManager_Open(argc, NULL);

		BufferSize= atoi(buff_size);

	CheckFiles(inputfile,outputfile);
	
	inFile=fopen(inputfile,"rb");
	outFile=fopen(outputfile,"wb");

    if (DSP_SUCCEEDED(status))
	 {
        /* Perform processor level initialization. */
        status = InitializeIVAProcessor(&ivaDmmTask,IvaImage);
    	 }

    if (DSP_SUCCEEDED(status)) 
	{

        /* Perform node level initialization. */
        status = InitializeIVANode(&ivaDmmTask);

        if (DSP_SUCCEEDED(status)) {

            /* Run task. */
            StopIVAProcessor(&ivaDmmTask);
           	status = RunIVATask(&ivaDmmTask, inFile, outFile,BufferSize,mapType);

            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "IVADMM:RunTask succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "IVADMM:RunTask failed.\n");
            }
        }
    }

    /* Close opened files. */
    if (inFile) {
        fclose(inFile);
    }

    if (outFile) {
        fclose(outFile);
    }

    CleanupIVANode(&ivaDmmTask);
    CleanupIVAProcessor(&ivaDmmTask);

            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "IVADMM:File Copy Success.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "IVADMM:File Copy Failed.\n");
            }

}

/*
 *  ======== InitializeIVAProcessor ========
 *  Perform processor related initialization.
 */
static DSP_STATUS InitializeIVAProcessor(IVAMSG_TASK *ivaDmmTask,char *IvaImage)
{
    DSP_STATUS          status = DSP_SOK;
    INT			argc = 1;
    INT 		index=0;
    INT 		procId;
    struct DSP_PROCESSORINFO   dspInfo;
    UINT                numProcs;

    /* Attach to an available DSP (in this case, the IVA). */


  while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
        {
                if (dspInfo.uProcessorType == IVA_ARM7)
                {
                        TSTLOG_appendLog(pHandle,"IVA device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                }
                index++;
        }



  /* Attach to IVA */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, NULL,&ivaDmmTask->hProcessorIva);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach failed. Status = 0x%x\n",
                status);
        }
    }
        else
       TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    
if (DSP_SUCCEEDED(status))
    {

            status = DSPProcessor_Load(ivaDmmTask->hProcessorIva,argc,&IvaImage,NULL) ;

            if (DSP_SUCCEEDED(status))
            {
                    TSTLOG_appendLog(pHandle, "IVADMM :Load succeeded.\n") ;
            }
            else
            {
                    TSTLOG_appendLog(pHandle, "IVADMM :Load failed.\n") ;
            }
            status = DSPProcessor_Start(ivaDmmTask->hProcessorIva) ;

            if (DSP_SUCCEEDED(status))
            {
                TSTLOG_appendLog(pHandle, "IVADMM:DSPProcessor_Start succeeded.\n") ;
            }
            else
            {
                TSTLOG_appendLog(pHandle, "IVADMM:DSPProcessor_Start failed.\n") ;
            }
		
	    status = DSPProcessor_Ctrl(ivaDmmTask->hProcessorIva,PROC_PWRMGT_ENABLE,NULL);
	    if (DSP_SUCCEEDED(status))
    	    {
        	TSTLOG_appendLog(pHandle, "DSPProcessor_Ctrl:PROC_PWRMGT_ENABLE succeeded.\n") ;
    	    }
    	    else
    	    {
        	TSTLOG_appendLog(pHandle, "DSPProcessor_Ctrl:PROC_PWRMGT_ENABLE failed.\n") ;
    	    }



    }

    return (status);
}


/*
 *  ======== InitializeIVANode ========
 *  Perform node related initialization.
 */
static DSP_STATUS InitializeIVANode(IVAMSG_TASK *ivaDmmTask)
{
    BYTE            argsBuf[ARGSIZE + sizeof(ULONG)];
    struct DSP_CBDATA      * pArgs;
    struct DSP_NODEATTRIN  nodeAttrInIva;
    struct DSP_UUID        uuid;
    DSP_STATUS      status = DSP_SOK;

    uuid = IVA_DMMCOPY_TI_uuid;

    nodeAttrInIva.uTimeout = DSP_FOREVER;
    nodeAttrInIva.iPriority = 5;

    pArgs = (struct DSP_CBDATA *)argsBuf;
    pArgs->cbData = ARGSIZE;

    /* Allocate the dmmcopy node. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Allocate(ivaDmmTask->hProcessorIva, &uuid, pArgs,&nodeAttrInIva,&ivaDmmTask->hNodeIva);

        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "IVANode_Allocate succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "IVANode_Allocate failed. Status = 0x%x\n",
                status);
        }
    }

    /* Create the dmmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Create(ivaDmmTask->hNodeIva);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "IVADMM:DSPNode_Create succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "IVADMM:DSPNode_Create failed. Status = 0x%x\n",
                status);
        }
    }

    /* Start the dmmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Run(ivaDmmTask->hNodeIva);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Run succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPNode_Run failed. Status = 0x%x\n",
                status);
        }
    }

    return (status);
}



/*Added by ramesh to check this code works for various models*/
static DSP_STATUS RunIVATask(IVAMSG_TASK *ivaDmmTask,FILE *inFile,FILE *outFile,INT size,MAPModel_t *mapModel)
{
    int				i;
    INT             cBytesRead = 0;
    DSP_STATUS      status = DSP_SOK;

    /* Actual MPU Buffer addresses */
    ULONG           aBufferSend = 0;
    ULONG           aBufferRecv = 0;

    /* Reserved buffer DSP virtual addresses (bytes) */
    ULONG           aDspSendBuffer = 0;
    ULONG           aDspRecvBuffer = 0;

    /* Base addresses of reserved DSP virtual address ranges (bytes) */
    ULONG           dspAddrSend = 0;
    ULONG           dspAddrRecv = 0;

    /* MPU buffer size */
    ULONG           ulSendBufferSize;
    ULONG           ulRecvBufferSize;

    /* DSP reserve size = buffersize + 4K page size */
	ulSendBufferSize=size;
	ulRecvBufferSize=size;
    ULONG           ulSendResv = ulSendBufferSize + 0x1000;
    ULONG           ulRecvResv = ulRecvBufferSize + 0x1000;

    /* Messaging used for GPP/DSP synchronization */
    struct DSP_MSG         msgToDsp;
    struct DSP_MSG         msgFromDsp;

    FILE            *test=NULL;
    int		    j;
    int             fd;

    if (mapModel == USER_VIRTUAL) {
        TSTLOG_appendLog(pHandle," Running DMM copy with user mallocs USER_VIRTUAL \n");
    /* Allocate MPU Buffers */
    aBufferSend = (ULONG) malloc(ulSendBufferSize);
    aBufferRecv = (ULONG) malloc(ulRecvBufferSize);
    }
    else {
        TSTLOG_appendLog(pHandle," Running DMM copy with mmap VAs \n");
        
        fd = open("/dev/mem",2);
        
        if (fd) {
            aBufferSend = (ULONG) mmap (aBufferSend, /* 0, */
                    ulSendBufferSize,
                    PROT_READ | PROT_WRITE,
                    MAP_SHARED,
                    fd,
                    PHYS_ADDR1);

            TSTLOG_appendLog(pHandle,"mapped aBufferSend VA=0x%x , PA=0x%x \n",aBufferSend,PHYS_ADDR1);

            aBufferRecv = (ULONG) mmap (aBufferRecv, /* 0, */
                    ulRecvBufferSize,
                    PROT_READ | PROT_WRITE,
                    MAP_SHARED,
                    fd,
                    PHYS_ADDR2);

            TSTLOG_appendLog(pHandle,"mapped aBufferRecv VA=0x%x \n, PA=0x%x \n",aBufferRecv,PHYS_ADDR2);
        }   
    } 

    if (!aBufferSend || !aBufferRecv) {
        TSTLOG_appendLog(pHandle, "Failed to allocate MPU buffers.\n");
        status = DSP_EMEMORY;
    } else {

        /* Reserve DSP virtual memory for send buffer */
        status = DSPProcessor_ReserveMemory(ivaDmmTask->hProcessorIva,ulSendResv, (PVOID *)&dspAddrSend);

        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_ReserveMemory succeeded , sendAddr = %x \n",dspAddrSend);
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_ReserveMemory failed. "
                    "Status = 0x%x\n", status);
        }
    }

    if (DSP_SUCCEEDED(status)) {

        /* Reserve DSP virtual memory for receive buffer */
        status = DSPProcessor_ReserveMemory(ivaDmmTask->hProcessorIva,ulRecvResv, (PVOID *)&dspAddrRecv);

        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_ReserveMemory succeeded , RecvAddr = %x \n",dspAddrRecv);
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_ReserveMemory failed. " "Status = 0x%x\n", status);
        }
    }

    if (DSP_SUCCEEDED(status)) {

        if (mapModel != PHYSICAL) {
            TSTLOG_appendLog(pHandle,"Mapping user Virtual to DSP virtual memory  \n");
            /* Map MPU "send buffer" to DSP "receive buffer" virtual address */
            status = DSPProcessor_Map(ivaDmmTask->hProcessorIva,(PVOID)aBufferSend, ulSendBufferSize,(PVOID)dspAddrSend, (PVOID *)&aDspRecvBuffer, DSP_MAPVIRTUALADDR);
        }
        else {
            TSTLOG_appendLog(pHandle,"Mapping user Physical memory to DSP virtual memory \n");
            /* Map MPU "send buffer" to DSP "receive buffer" virtual address */
            status = DSPProcessor_Map(ivaDmmTask->hProcessorIva,
                    (PVOID)PHYS_ADDR1, ulSendBufferSize,
                    (PVOID)dspAddrSend, (PVOID *)&aDspRecvBuffer, DSP_MAPPHYSICALADDR | DSP_MAPELEMSIZE8);
            /* note -- For pre3.31 releases, just use 'DSP_MAPELEMSIZE8' as
             * map attribute */ 
        }

        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Map succeeded , aDspRecvBuffer = %x \n",aDspRecvBuffer);
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Map failed. "
                    "Status = 0x%x\n", status);
        }
    }

    if (DSP_SUCCEEDED(status)) {

        if (mapModel != PHYSICAL) {
           TSTLOG_appendLog(pHandle, "Mapping user Virtual to DSP virtual memory \n"); 
        /* Map MPU "receive buffer" to DSP "send buffer" virtual address */
        status = DSPProcessor_Map(ivaDmmTask->hProcessorIva,
                (PVOID)aBufferRecv, ulRecvBufferSize,
                (PVOID)dspAddrRecv, (PVOID *)&aDspSendBuffer, 0);
        }
        else {
           TSTLOG_appendLog(pHandle, "Mapping user Physical memory to DSP virtual memory \n"); 
        /* Map MPU "receive buffer" to DSP "send buffer" virtual address */
        status = DSPProcessor_Map(ivaDmmTask->hProcessorIva,
                (PVOID)PHYS_ADDR2, ulRecvBufferSize,
                (PVOID)dspAddrRecv, (PVOID *)&aDspSendBuffer, DSP_MAPPHYSICALADDR | DSP_MAPELEMSIZE8);
            /* note -- For pre3.31 releases, just use 'DSP_MAPELEMSIZE8' as
             * map attribute */ 
        }

        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Map succeeded , aDspSendBuffer = %x \n",aDspSendBuffer);
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Map failed. "
                    "Status = 0x%x\n", status);
        }
    }

    if (DSP_SUCCEEDED(status)) {

        /* Notify DSP of word-adjusted buffer addresses */
        msgToDsp.dwCmd = DMM_SETUPBUFFERS;
        msgToDsp.dwArg1 = (DWORD)aDspRecvBuffer + IVA_VAOFFSET;
        msgToDsp.dwArg2 = (DWORD)aDspSendBuffer + IVA_VAOFFSET;

        status = DSPNode_PutMessage(ivaDmmTask->hNodeIva, &msgToDsp, DSP_FOREVER);
        if (DSP_SUCCEEDED(status))
        {
            TSTLOG_appendLog(pHandle,"Sent DMM<_SETUP buff to IVA: fromGPP=%x , toGPP=%x \n",
                    msgToDsp.dwArg1,msgToDsp.dwArg2);
        }
        else
        {
            TSTLOG_appendLog(pHandle, "DSPProcessor_PutMessage failed. "
                    "Status = 0x%x\n", status);
        }
    }

    if (DSP_SUCCEEDED(status)) {

        /*
         * Read a fixed-size buffer from a data stream,
         * copy data to output buffer, and read back from
         * input buffer.
         */
           /* do a flush of the buffers, before using them */
        DSPProcessor_FlushMemory(ivaDmmTask->hProcessorIva,
                (PVOID)aBufferSend, DEFAULTBUFSIZE, 0);

        DSPProcessor_FlushMemory(ivaDmmTask->hProcessorIva,
                (PVOID)aBufferRecv, DEFAULTBUFSIZE, 0);
        do {
            /* Read a block of data from the input file */
            cBytesRead = fread((PSTR)aBufferSend, sizeof(BYTE),
                    DEFAULTBUFSIZE, inFile);

            if (cBytesRead > 0) {


                /* Go ahead and flush here */
                DSPProcessor_FlushMemory(ivaDmmTask->hProcessorIva,
                        (PVOID)aBufferSend, DEFAULTBUFSIZE, 0);

                /* Tell DSP how many words to read */
                msgToDsp.dwCmd = DMM_WRITEREADY;
                msgToDsp.dwArg1 = (DWORD)cBytesRead;

                status = DSPNode_PutMessage(ivaDmmTask->hNodeIva, &msgToDsp,
                        DSP_FOREVER);

                if (DSP_FAILED(status)) {

                    TSTLOG_appendLog(pHandle, "DSPProcessor_PutMessage failed. "
                            "Status = 0x%x\n", status);
                } else {

                    /*
                     * Wait to hear back from DSP that it has finished
                     * copying the buffers and that it's okay for us
                     * to read buffer again
                     */
                    status = DSPNode_GetMessage(ivaDmmTask->hNodeIva,
                            &msgFromDsp, DSP_FOREVER);

                    if (DSP_SUCCEEDED(status)) {
                        /* Go ahead and flush here */
                        if (msgFromDsp.dwCmd == DMM_WRITEREADY)
                        { 
                            DSPProcessor_FlushMemory(ivaDmmTask->hProcessorIva,
                                    (PVOID)aBufferRecv, DEFAULTBUFSIZE, 0);

                            for (j=0;j<cBytesRead;j++)
                            {
                                if (((unsigned char*)aBufferSend)[j] != ((unsigned char*)aBufferRecv)[j])
                                {	
                                    TSTLOG_appendLog(pHandle, "Sent and received buffers do n't match \n");
                                    TSTLOG_appendLog(pHandle, "Press any key to continue \n");
                                    (void) getchar();
                                    break;
                                }	
                            }
                            /*
                             * Copy the block of data from the DSP into
                             * the output file
                             */
                            fwrite((PSTR)aBufferRecv, sizeof(BYTE),
                                    cBytesRead, outFile);
                        } 
                    }
                    else {

                        TSTLOG_appendLog(pHandle, "DSPProcessor_GetMessage failed. "
                                "Status = 0x%x\n", status);
                    }
                }
            }
        } while ((cBytesRead > 0) && DSP_SUCCEEDED(status));
    }
    /* Unmap DSP Recv buffer from MPU receive buffer */
    status = DSPProcessor_UnMap(ivaDmmTask->hProcessorIva,
            (PVOID)aDspRecvBuffer);

    if (DSP_SUCCEEDED(status)) {
        TSTLOG_appendLog(pHandle, "DSPProcessor_UnMap succeeded.\n");
    }
    else {
        TSTLOG_appendLog(pHandle, "DSPProcessor_UnMap failed. "
                "Status = 0x%x\n", status);
    }

    /* Upmap DSP Send buffer from MPU receive buffer */
    status = DSPProcessor_UnMap(ivaDmmTask->hProcessorIva,
            (PVOID)aDspSendBuffer);

    if (DSP_SUCCEEDED(status)) {
        TSTLOG_appendLog(pHandle, "DSPProcessor_UnMap succeeded.\n");
    }
    else {
        TSTLOG_appendLog(pHandle, "DSPProcessor_UnMap failed. "
                "Status = 0x%x\n", status);
    }

    /* Unreserve DSP virtual memory */
    status = DSPProcessor_UnReserveMemory(ivaDmmTask->hProcessorIva,
            (PVOID)dspAddrSend);

    if (DSP_SUCCEEDED(status)) {
        TSTLOG_appendLog(pHandle, "DSPProcessor_UnReserveMemory succeeded.\n");
    }
    else {
        TSTLOG_appendLog(pHandle, "DSPProcessor_UnReserveMemory failed. "
                "Status = 0x%x\n", status);
    }

    /* Unreserve DSP virtual memory */
    status = DSPProcessor_UnReserveMemory(ivaDmmTask->hProcessorIva,
            (PVOID)dspAddrRecv);

    if (DSP_SUCCEEDED(status)) {
        TSTLOG_appendLog(pHandle, "DSPProcessor_UnReserveMemory succeeded.\n");
    }
    else {
        TSTLOG_appendLog(pHandle, "DSPProcessor_UnReserveMemory failed. "
                "Status = 0x%x\n", status);
    }
    
    if (mapModel == USER_VIRTUAL) {
        TSTLOG_appendLog(pHandle," Running DMM copy with user mallocs USER_VIRTUAL \n");

        /* Free MPU buffers (at originally allocated addresses) */
        if (aBufferSend) {
            free((VOID *)aBufferSend);
        }

        if (aBufferRecv) {
            free((VOID *)aBufferRecv);
        }
    }
    else
    {
        munmap(aBufferSend,ulSendBufferSize);
        munmap(aBufferRecv,ulRecvBufferSize);
        close(fd); 
    }

    return (status);
}


/*
 *  ======== CleanupIVANode ========
 *  Perform node related cleanup.
 */
static DSP_STATUS CleanupIVANode(IVAMSG_TASK *ivaDmmTask)
{
    DSP_STATUS      status;
    DSP_STATUS      exitStatus;

    if (ivaDmmTask->hNodeIva)
	 {

        /* Terminate DSP node */
        	status = DSPNode_Terminate(ivaDmmTask->hNodeIva, &exitStatus);
	        if (DSP_SUCCEEDED(status)) 
		{
	            TSTLOG_appendLog(pHandle, "IVADMM:DSPNode_Terminate succeeded.\n");
        	}
        	else 
		{
	            TSTLOG_appendLog(pHandle, "IVADMM:DSPNode_Terminate failed: 0x%x\n", status);
        	}

        /* Delete DSP node */
	        status = DSPNode_Delete(ivaDmmTask->hNodeIva);
        	if (DSP_SUCCEEDED(status))
		 {
	            TSTLOG_appendLog(pHandle, "DSPNode_Delete succeeded.\n");
        	 }
        	else
		 {
            	   TSTLOG_appendLog(pHandle, "DSPNode_Delete failed: 0x%x\n", status);
        	 }

        	ivaDmmTask->hNodeIva = NULL;
        }

    return (status);
}


/*
 *  ======== CleanupIVAProcessor ========
 *  Perform processor related cleanup.
 */
static DSP_STATUS StopIVAProcessor(IVAMSG_TASK *ivaDmmTask)
{
    	DSP_STATUS      status;
	status = DSPProcessor_Stop(ivaDmmTask->hProcessorIva) ;
        if (DSP_SUCCEEDED(status))
        {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Stop:IVA succeeded.\n") ;
        }
        else
        {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Stop:IVA failed.\n") ;
        }
	return (status);


}
static DSP_STATUS CleanupIVAProcessor(IVAMSG_TASK *ivaDmmTask)
{
    DSP_STATUS      status;

    if (ivaDmmTask->hProcessorIva) 
   {
/*	status = DSPProcessor_Stop(ivaDmmTask->hProcessorIva) ;
        if (DSP_SUCCEEDED(status))
        {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Stop:IVA succeeded.\n") ;
        }
        else
        {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Stop:IVA failed.\n") ;
        }*/

        /* Detach from processor. */
        status = DSPProcessor_Detach(ivaDmmTask->hProcessorIva);
        if (DSP_SUCCEEDED(status)) 
	{
            TSTLOG_appendLog(pHandle, "IVADMM:DSPProcessor_Detach succeeded.\n");
        }
        else 
	{
            TSTLOG_appendLog(pHandle, "IVADMM:DSPProcessor_Detach failed.\n");
        }

        ivaDmmTask->hProcessorIva = NULL;
    }

    return (status);
}



/*
 *  ======== CheckFiles ========
 *  To check the integrity of the files supplied.
 *  
 */


static DSP_STATUS CheckFiles(char* inFile,char* outFile)
{

	FILE 		*infd;
	FILE 		*outfd;
    	DSP_STATUS      status = DSP_EFAIL;
	
	infd=fopen(inFile,"rb");
	
	if(infd)
	{
	   outfd=fopen(outFile,"wb");
		if(outfd)	
		{
		  status=DSP_SOK;	
		}
		else
		{	
		  TSTLOG_appendLog(pHandle,"%s:Unable to open file for writing\n",outFile);
		  fclose(infd);
		}	
	}
	else
	{
            TSTLOG_appendLog(pHandle,"%s: Unable to open file for reading\n",inFile);

	}
	fclose(infd);
	fclose(outfd);
  	
    return(status);
}

