/*
 *  ======== ping.c ========
 *  Description:
 *     Console application to manage the DSP/BIOS Bridge Ping Task Node.
 *
 *  Demonstrates:
 *      1. How to create a task node with initial arguments.
 *      2. How to get notified of, and retrieve messages, sent from a DSP node.
 *
 *  Requires:
 *      1. Node PING_TI must be configured into the base image (via Gconf)
 *
 *  Usage:
 *      ping.out <message_count> [<delay_in_milliseconds>]
 *
 *  Notes:
 *
 */

#include <stdio.h>
#include <getopt_test.h>
#include <dbapi.h>
#include <tst.h>
#include <tstlog.h>
#include <DSPProcessor_OEM.h>
#include <testosal.h>
#include <string.h>

#define DEFAULTMSGS   50            /* default message count                 */
#define DEFAULTDELAY  "50"          /* default ping delay                    */
#define DSPUNIT       0             /* Default (first) DSP ID                */
#define ARGSIZE       32            /* Size of arguments to Ping node.       */
#define MAXNAMELEN    64            /* Max length of event name.             */
#define MAXMSGLEN     128           /* Max length of MessageBox msg.         */
#define PING 4
#define DSPEXEC "iva.dof"
#define DEFNAMESIZE   80
static CONST CHAR          *aArgv[80] = { DSPEXEC, "arg1" };
static const struct DSP_UUID PING_TI_uuid = {
        0x6562df2a, 0x7068, 0x42ef, 0x84, 0x59, {
        0x75, 0x58, 0x4c, 0x1c, 0xdb, 0x8f
    }
};
static FILE                             *pHandle;
static BOOL                             fLoadDsp = FALSE, fOpenLog = FALSE;;
static CHAR                             szLogName[DEFNAMESIZE];

/* Ping task context data structure - passed between functions in this module */
typedef struct {
    DSP_HPROCESSOR  hProcessor;     /* Handle to processor.             */
    DSP_HNODE       hNode;          /* Handle to node.                  */
    HANDLE          hEvent;         /* Event to be signalled by DSP     */
    UINT            msgCount;       /* Number of messages DSP sends     */
} PING_TASK;

/* Argument structure for DSPNode_Allocate():                           */
typedef struct {
    ULONG   cbData;
    BYTE    cData[ARGSIZE];
} PING_NODEDATA;

/* Forward Declarations */
static DSP_STATUS ProcessArgs        (int argc, char ** argv, UINT * pMsgCnt,
                                      PING_NODEDATA *argsBuf);
static DSP_STATUS AttachProcessor    (PING_TASK *pingTask);
static DSP_STATUS InitializeProcessor(PING_TASK *pingTask);
static DSP_STATUS CreateNode         (PING_TASK *pingTask,
                                      PING_NODEDATA *argsBuf);
static DSP_STATUS RunNode            (PING_TASK *pingTask);
static DSP_STATUS DestroyNode        (PING_TASK *pingTask);
static DSP_STATUS DetachProcessor    (PING_TASK *pingTask);


/*
 *  ======== main ========
 */
int IvaPing(int argc, char ** argv)
{
    PING_TASK     pingTask;           /* Ping task context                    */
    UINT          msgCount;           /* Number of messages DSP sends         */
    PING_NODEDATA argsBuf;            /* Task Node args.                      */
    DSP_STATUS    status = DSP_SOK;
    INT             opt = 0;
    fLoadDsp = FALSE;


    /* Process command line arguments, open data files: */
    optind = 0;
       while ((opt = getopt_test(argc, (char * const)argv, "+vl:x:n:s:ZPR:")) != EOF) {
        switch (opt) {
         case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;

                case 'l':
            /* use log and store with name location */
                        strncpy(szLogName, optarg, sizeof(szLogName));
                        pHandle = TSTLOG_openLog(szLogName);
                        fOpenLog = TRUE;
                        TSTLOG_appendLog(pHandle, "Log File Set to %s\n:", szLogName);
                        break;

                case 'x':
            /* use given executable */
                        aArgv[0] = (char *) optarg;
                        fLoadDsp = TRUE;
                        TSTLOG_appendLog(pHandle, "New Executable %s\n:", aArgv[0]);
                        break;


	        default:
        		TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
                        break;
        }
	}






    ///status = ProcessArgs(argc, argv, &msgCount, &argsBuf);
    if (DSP_SUCCEEDED(status)) {

        /* Initialize context: */
        pingTask.hProcessor = NULL;
        pingTask.hNode = NULL;
        pingTask.hEvent = NULL;
        pingTask.msgCount = msgCount ;

        status = DspManager_Open (0, NULL);

        if (DSP_SUCCEEDED(status))
        {

            /* Perform processor level initialization. */
            status = AttachProcessor(&pingTask);
            if (DSP_SUCCEEDED(status)) {

		 status = InitializeProcessor(&pingTask);

              /* Perform node level initialization. */
              status = CreateNode(&pingTask, &argsBuf);
		TSTLOG_appendLog(pHandle," Calling RunNode function and the status is 0x%x\n",status);
              if (DSP_SUCCEEDED(status)) {
                  /* Run Ping task. */
                 status = RunNode(&pingTask);
              }
            }

            /* Cleanup all allocated resources. */
            DestroyNode(&pingTask);
            DetachProcessor(&pingTask);
        }
    }
    status = DspManager_Close(0,NULL) ;

    return (DSP_SUCCEEDED(status) ? 0 : -1);
}


/*
 *  ======== AttachProcessor ========
 *  Perform processor related initialization.
 */
static DSP_STATUS AttachProcessor(PING_TASK *pingTask)
{
    DSP_STATUS      	status = DSP_EFAIL;
	struct DSP_PROCESSORINFO	dspInfo;
	UINT				numProcs;
	UINT				index=0;
	INT					procId;

	while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
	{
		if (dspInfo.uProcessorType == IVA_ARM7)
		{
			TSTLOG_appendLog(pHandle,"IVA device detected !! \n");
			procId = index;
			status = DSP_SOK;	
			break;	
		}
		index++;	
	}

    /* Attach to a DSP (first by default) */
    status = DSPProcessor_Attach(procId, NULL, &pingTask->hProcessor);
    if (DSP_FAILED(status)) {
       TSTLOG_appendLog(pHandle, "IVADSPProcessor_Attach failed: 0x%x\n", status);
    }

    return (status);
}

static DSP_STATUS InitializeProcessor(PING_TASK *pingTask)
{
        DSP_STATUS          status = DSP_SOK;
        INT                 iArgc = 1 ;
                status = DSPProcessor_Load(pingTask->hProcessor, iArgc, aArgv,NULL);
                if (DSP_SUCCEEDED(status))
                {
                                TSTLOG_appendLog(pHandle, "DSPProcessor_Load succeeded\n");
                        status = DSPProcessor_Start(pingTask->hProcessor);
                        if (DSP_FAILED(status))
                        {
                                        TSTLOG_appendLog(pHandle, "Error in DSPProcessor Start.Status = 0x%x\n", status);
                        }
			else
			{
                                        TSTLOG_appendLog(pHandle, "DSPProcessor Start Success .Status = 0x%x\n", status);

			}

                }
                else
                {
                                TSTLOG_appendLog(pHandle,"Error in DSPProcessor_Load. Status = 0x%x\n",status);
                }
        return (status);
}


/*
 *  ======== CreateNode ========
 *  Perform node related initialization.
 */
static DSP_STATUS CreateNode(PING_TASK *pingTask, PING_NODEDATA *argsBuf)
{
    struct DSP_CBDATA          *pArgs;
    DSP_STATUS          status;

    pArgs = (struct DSP_CBDATA *)argsBuf;
    struct DSP_NODEATTRIN  nodeAttrInIva;


    /* Allocate the ping node, passing arguments for node create phase. */
    status = DSPNode_Allocate(pingTask->hProcessor, &PING_TI_uuid, pArgs,&nodeAttrInIva, &pingTask->hNode);
// status = DSPNode_Allocate(copyTask->hProcessorIva, &IVA_PING_TI_uuid, pArgsDsp,&nodeAttrInIva, &copyTask->hNodeIva);

    if (DSP_FAILED(status)) {
       TSTLOG_appendLog(pHandle, "DSPNode_Allocate failed: 0x%x\n", status);
    }

    /* Create the ping node on the DSP. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Create(pingTask->hNode);
        if (DSP_FAILED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Create failed: 0x%x\n", status);
        }
    }

    /* Register for message notifications:  */
    if (DSP_SUCCEEDED(status)) {

        pingTask->hEvent = OsSpecificMemAlloc(sizeof(struct DSP_NOTIFICATION));

        status = DSPNode_RegisterNotify(pingTask->hNode,DSP_NODEMESSAGEREADY,DSP_SIGNALEVENT, pingTask->hEvent);
        if (DSP_FAILED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_RegisterNotify failed: 0x%x\n", status);
        }
	else
	{
            TSTLOG_appendLog(pHandle, "DSPNode_RegisterNotify is Success: 0x%x\n", status);

	}
    }

    return (status);
}


/*
 *  ======== RunNode ========
 *  Run ping task.
 */
static DSP_STATUS RunNode(PING_TASK *pingTask)
{
    DSP_STATUS          status;
    DSP_STATUS          exitStatus;
    BOOL                fDone = FALSE;
    struct DSP_MSG             dspMsg;
    UINT                uIndex;
    DWORD               cEvents;
    struct DSP_NOTIFICATION*   ahEvents[2];

    cEvents     = 1;
    ahEvents[0] = pingTask->hEvent;
    int                 flag=0;
    /* Start the ping node on the DSP. */

	dspMsg.dwArg1=3;
	dspMsg.dwArg2=0;
	dspMsg.dwCmd = 7;
    status = DSPNode_Run(pingTask->hNode);
       TSTLOG_appendLog(pHandle, "DSPNode_Run called and status is : 0x%x\n", status);
    if (DSP_FAILED(status)) {
       TSTLOG_appendLog(pHandle, "DSPNode_Run failed: 0x%x\n", status);
    }

    if (DSP_SUCCEEDED(status)) {
        /* Display DSP messages until Ping MessageBox dismissed: */
        //do {
            /* Block until pinged by DSP or key event: */
            status = DSPNode_PutMessage(pingTask->hNode, &dspMsg, 1500);
	    TSTLOG_appendLog(pHandle,"After 1st PutMessage status is 0x%x\n",status);
	    dspMsg.dwArg1=2;
	    dspMsg.dwArg2=0;
	    dspMsg.dwCmd = 7;
            status = DSPNode_PutMessage(pingTask->hNode, &dspMsg, 1500);
	    TSTLOG_appendLog(pHandle,"After 2nd PutMessage status is 0x%x\n",status);
	    dspMsg.dwArg1=1;
	    dspMsg.dwArg2=0;
	    dspMsg.dwCmd = 7;
            status = DSPNode_PutMessage(pingTask->hNode, &dspMsg, 1500);
	    TSTLOG_appendLog(pHandle,"After 3rd PutMessage Before calling DSPManager_WaitForEvents status is 0x%x\n",status);
            ///status = DSPManager_WaitForEvents(ahEvents, cEvents,&uIndex,100000);
            status = DSPManager_WaitForEvents(ahEvents, cEvents,&uIndex,DSP_FOREVER);
	   
	    TSTLOG_appendLog(pHandle,"DSPManager_WaitForEvents status is 0x%x\n",status);
	   if(flag==0)
	    {
	      flag=1;
	    }

            /* Process events: */
            if (DSP_SUCCEEDED(status) && uIndex == 0)
            {
                /* DSP signalled us: */
                while (DSP_SUCCEEDED(status)) {
		    TSTLOG_appendLog(pHandle,"Waiting in main while looop !! Please wait!! and uIndex is %d\n",uIndex);
                    status = DSPNode_GetMessage(pingTask->hNode, &dspMsg, 1500);
		    TSTLOG_appendLog(pHandle,"After Calling GetMsg main while looop and status is 0x%x !!\n",status);
                    if (DSP_SUCCEEDED(status)) {
                        TSTLOG_appendLog(pHandle, "Ping: Id %2d Msg %3d Mem %4d\n",dspMsg.dwCmd, dspMsg.dwArg1, dspMsg.dwArg2);

                       /* if (dspMsg.dwArg1 >= pingTask->msgCount)
                        {
                            fDone = TRUE;
                            break;
                        }*/
                    }
                }

                if (DSP_FAILED(status)) {
                    if (status == DSP_ETIMEOUT) {
			TSTLOG_appendLog(pHandle,"Entered in the state of timeout and the status is 0x%x\n",status);
                        /* Okay to timeout if message queue is empty */
                        //status = DSP_SOK;
                        //continue;
                    }
                    else {
                        TSTLOG_appendLog(pHandle, "DSPNode_GetMessage failed: 0x%x\n",status);
                    }
                }
            }
            else {
                /* Wait error: */
                TSTLOG_appendLog(pHandle,
                    "DSPManager_WaitForBridgeEvents Failed: 0x%x\n", status);
                fDone = TRUE;
            }

       /// } while (DSP_SUCCEEDED(status));  //while (!fDone && DSP_SUCCEEDED(status));
    }

    /* Terminate the DSP node. */
    status = DSPNode_Terminate(pingTask->hNode, &exitStatus);
    if (DSP_FAILED(status)) {
        TSTLOG_appendLog(pHandle, "DSPNode_Terminate failed: 0x%x\n", status);
    }

    return (status);
}


/*
 *  ======== DestroyNode ========
 *  Perform node related cleanup.
 */
static DSP_STATUS DestroyNode(PING_TASK *pingTask)
{
    DSP_STATUS      status = DSP_SOK;

    if (pingTask->hNode) {
        /* Delete DSP node. */
        status = DSPNode_Delete(pingTask->hNode);
        if (DSP_FAILED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Delete failed: 0x%x\n", status);
        }

        pingTask->hNode = NULL;
    }

    if (pingTask->hEvent) {
        OsSpecificMemFree(pingTask->hEvent);
    }

    return (status);
}


/*
 *  ======== DetachProcessor ========
 *  Perform processor related cleanup.
 */
static DSP_STATUS DetachProcessor(PING_TASK *pingTask)
{
    DSP_STATUS      status = DSP_SOK;

    if (pingTask->hProcessor) {
        /* Detach from processor. */
        status = DSPProcessor_Detach(pingTask->hProcessor);
        if (DSP_FAILED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Detach failed: 0x%x\n", status);
        }

        pingTask->hProcessor = NULL;
    }

    return (status);
}


/*
 *  ======== ProcessArgs ========
 *  Process command line arguments for this sample, returning input and
 *  output file handles.
 */
static DSP_STATUS ProcessArgs(int argc, char ** argv, UINT * pMsgCnt,PING_NODEDATA *argsBuf)
{
    DSP_STATUS  status = DSP_SOK;

    switch (argc) {
    case 1:
        strncpy(argsBuf->cData, DEFAULTDELAY, ARGSIZE);
        *pMsgCnt = DEFAULTMSGS;
        break;

    case 2:
        sscanf(argv[1], "%u", pMsgCnt);
        strncpy(argsBuf->cData, DEFAULTDELAY, ARGSIZE);
        break;

    case 3:
        sscanf(argv[1], "%u", pMsgCnt);
        strncpy(argsBuf->cData, argv[2], ARGSIZE);
        argsBuf->cData[ARGSIZE-1] = '\0';
        break;

    default:
        TSTLOG_appendLog(pHandle,
            "Usage: %s <message_count> [<delay_in_milliseconds>] \n", argv[0]);
        status = DSP_EFAIL;
        break;
    }

    if (DSP_SUCCEEDED(status)) {
        argsBuf->cbData = strlen(argsBuf->cData) + 1;
    }
    return (status);
}
