/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    dspprocst.c
 *
 *   Path    $(TESTSUITEROOT)\Behaviour\DSPProcState
 * 
 *   Desc:
 *        This module tests the state of a processor following various
 *        DSPProcessor API function calls.  This test allows various
 *        APIs which alter the state of the Processor.  A Processor
 *        is attached and various function calls that alter the state are
 *        issued by the user in a given sequence to verify the correct
 *        board transitions.  If the user supplied sequence is
 *        incorrect, then one function call may result in a failure.
 *        If DSPProcessor_Attach is not called as the first function,
 *        the sequence will fail.
 *
 *  Additional Note : State should change when appropriate API is called
 *			  and it should not change when other allowed APIs are called.
 *
 *  Usage:
 *        dspprocst [-v] [-l "log path/filename"]
 *                       [-x "valid executable" ]
 *                       [-p "processor number" ]
 *                       [-t "state transitions"]
 *
 *        -v: verbose mode
 *        -l: use output log and location
 *        -x: name of valid DSP executable
 *        -p: valid processor number (default 0)
 *        -t: input of state transition sequence
 *
 *  API Codes to use to specify sequence as input:
 *
 *         1: DSPProcessor_Attach
 *         2: DSPProcessor_Detach
 *         3: DSPProcessor_EnumNodes
 *         4: DSPProcessor_GetResourceInfo
 *         5: DSPProcessor_GetState
 *         6: DSPProcessor_RegisterNotify
 *         7: DSPProcessor_Load
 *         8: DSPProcessor_Start
 *         9: DSPProcessor_Ctrl
 *
 *  Example:
 *  framework BEHAVIOUR PROCST -v -l output.txt -x apitest0_nortdxidl.dof -t "1 2"
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL 03, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <dbapi.h>
#include <string.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>   
#include <DSPProcessor_OEM.h>

#define DEFAULTDSPUNIT      0
#define DEFNAMESIZE         80
#define DSPEXEC             "apitest0_nortdxidl.dof"

#define DEFSTATES           "1 2"
#define SEPARATORS          " , \t\n"

INT checkTransProc(const INT iPrevState, const INT iTransition);

/*
 *  ======== dspprocst_main ========
 *
 */
INT dspprocst_main (INT argc, CHAR * argv[])
{

    INT                     opt;
    INT                     cRetVal = 1;
    INT                     iPrevState;
    INT                     iTransition = 0;
    INT                     iArgc = 1;
    INT                     iProc = 0;
    UINT		            *puNumNodes, *puAllocated;
    BOOL                    bUseLogFile=FALSE;
    FILE                    *pHandle = NULL;
    CHAR                    szLogName[DEFNAMESIZE];
    CHAR                    *szErrorMsg = "";
    CONST CHAR              *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    CONST CHAR              *aEnvp[DEFNAMESIZE] = {NULL};
    CHAR		            *szProcStates;
    DSP_HPROCESSOR          hProcessor;
    DSP_STATUS              status;
    DSP_STATUS              exitStatus = DSP_SOK;
    struct DSP_PROCESSORATTRIN     *pAttrIn;
    struct DSP_CBDATA              *pArgs;
    DSP_HNODE               *aNodeTab;
    DSP_HNODE               hNode;
    struct DSP_UUID                gidNodeID;
    CHAR                    *pszUuid;
    struct DSP_NOTIFICATION        hNotification;
    CHAR                    *pstrList[50];
    INT                     cListsize = 0;
    INT                     count = 0;
    struct DSP_PROCESSORSTATE      *procState;
    struct DSP_RESOURCEINFO        rsInfo;
    
    pArgs       = OsSpecificMemAlloc(sizeof(struct DSP_CBDATA));
	pAttrIn     = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
	procState   = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORSTATE));
    puNumNodes  = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
	puAllocated = OsSpecificMemAlloc(sizeof(UINT));
    aNodeTab    = OsSpecificMemAlloc(sizeof(DSP_HNODE));
	   
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Set default set of state transitions */
    szProcStates = (char *)DEFSTATES;

	optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:p:x:t:")) != EOF) 
	{
        switch (opt) 
		{
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
        	bUseLogFile = TRUE;
            pHandle = TSTLOG_openLog(szLogName);
            break;
        case 'p':
            /* Set new processor number to test */
            iProc = CSL_Atoi(optarg);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            break;
        case 't':
            /* input state transitions */
            szProcStates = (char *) optarg;
            if (bUseLogFile) 
			{
                TSTLOG_appendLog(pHandle, "Using board states: \"%s\"\n",
                                 szProcStates);
            }
            TST_verbose("Using board states: \"%s\"\n", szProcStates);
            break;
        default:
            TST_verbose(("Invalid option: %c\n"), opt);
            cRetVal = 0;
            szErrorMsg = "Invalid option specified.";
            break;
        }
    }

    /* Attach to processor */
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId=0;
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, pAttrIn, &hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    }
    
    /* Set initial state */
    status = DSPProcessor_GetState(hProcessor, procState, sizeof(struct DSP_PROCESSORSTATE));

    if (DSP_FAILED(status)) 
	{
        cRetVal = 0;
        szErrorMsg = "Initial DSPProcessor_GetState() failed";
    }

    status = DSPProcessor_Detach(hProcessor);

    if (DSP_FAILED(status)) 
	{
        cRetVal = 0;
        szErrorMsg = "Initial DSPProcessor_Detach() failed";
    }

    if (cRetVal > 0) 
	{
    TST_verbose("Initiating DSPProcessor state testing\n");

    /* Process API list */
/*        pstrList[cListsize++] = strtok(szProcStates, SEPARATORS);
    while ((pstrList[cListsize++] = strtok(NULL, SEPARATORS)) != NULL);*/

    /* Adjust for NULL element */
    cListsize--;

    while (cRetVal > 0 && count < cListsize) 
	{
        switch(pstrList[count++][0]) 
		{
            case '1':
                /* DSPProcessor_Attach */
                iTransition = 1;
                status = DSPProcessor_Attach(iProc, pAttrIn, &hProcessor);

                szErrorMsg = "Error in DSPProcessor_Attach()";
                break;
            case '2':
				/* DSPProcessor_Detach */
                iTransition = 2;
                status = DSPProcessor_Detach(hProcessor);
                szErrorMsg = "Error in DSPProcessor_Detach()";
                break;
            case '3':
				/* DSPProcessor_EnumNodes */
                iTransition = 3;

				/* Allocate, Create, and Run a node */
				pszUuid = "7EEB2C7E_785A_11D4_A650_00C04F0C04F3";
                UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

                /* Allocate Node */
                status = DSPNode_Allocate(hProcessor, &gidNodeID, NULL, NULL,
                                     &hNode);
                /* Create Node on DSP */
                status = DSPNode_Create(hNode);
                /* Start Node running on DSP */
                status = DSPNode_Run(hNode);
                status = DSPProcessor_EnumNodes(hProcessor, aNodeTab, 50,
                         puNumNodes, puAllocated);
                szErrorMsg = "Error in DSPProcessor_EnumNodes()";

				DSPNode_Terminate(hNode, &exitStatus);
				DSPNode_Delete(hNode);
                break;
            case '4':
				/* DSPProcessor_GetResourceInfo */
                iTransition = 4;
                status = DSPProcessor_GetResourceInfo(hProcessor, 0,
                                   &rsInfo, sizeof(struct DSP_RESOURCEINFO));
                szErrorMsg = "Error in DSPProcessor_GetResourceInfo()";
                break;
            case '5':
				/* DSPProcessor_GetState */
                iTransition = 5;
                status = DSPProcessor_GetState(hProcessor, procState,
                                               sizeof(struct DSP_PROCESSORSTATE));
                szErrorMsg = "Error in DSPProcessor_GetState()";
                break;
            case '6':
				/* DSPProcessor_RegisterNotify */
                iTransition = 6;
                status = DSPProcessor_RegisterNotify(hProcessor,
                DSP_PROCESSORSTATECHANGE, DSP_SIGNALEVENT, &hNotification);

                szErrorMsg = "Error in DSPProcessor_RegisterNotify()";

                break;
            case '7':
				/* DSPProcessor_Load */
                iTransition = 7;
                status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, 
								(CONST CHAR **)aEnvp);
                szErrorMsg = "Error in DSPProcessor_Load()";
                break;
            case '8':
				/* DSPProcessor_Start */
                iTransition = 8;
                status = DSPProcessor_Start(hProcessor);
                szErrorMsg = "Error in DSPProcessor_Start()";
                break;
            case '9':
				/* DSPProcessor_Ctrl */
                iTransition = 9;
                status = DSPProcessor_Ctrl(hProcessor, 0x8010, pArgs);
                szErrorMsg = "Error in DSPProcessor_Ctrl()";
                break;
            default: TST_verbose("Invalid API call: %c\n", pstrList[cListsize-1][0]);
                break;
        } /* End of Switch Stmt */

        if (DSP_FAILED(status)) 
		{
            cRetVal = 0;
            break;
        }

       /* Get state and check for correctness, unless detached */
       if (iTransition != 2) 
	   {
          iPrevState = procState->iState;
          status = DSPProcessor_GetState(hProcessor, procState,
                                         sizeof(struct DSP_PROCESSORSTATE));
       }
       else 
	   {
          continue;
       }

       if (DSP_FAILED(status)) 
	   {
           cRetVal = 0;
           szErrorMsg = "Error in DSPProcessor_GetState() _\n";
           TST_verbose("%x", status);
           break;
       }
       else 
	   {
           /* Check to see if we are in the correct state */
           switch(procState->iState) 
		   {
           case PROC_STOPPED:
               if (checkTransProc(iPrevState, iTransition) != PROC_STOPPED) 
			   {
                   cRetVal = 0;
                   TSTLOG_appendLog(pHandle,
                            "\nTransition %i from state %i to %i invalid.\n",
                            iTransition, iPrevState, procState->iState);
               }
               break;
           case PROC_LOADED:
               if (checkTransProc(iPrevState, iTransition) != PROC_LOADED) 
			   {
                   cRetVal = 0;
                   TSTLOG_appendLog(pHandle,
                           "\nTransition %i from state %i to %i invalid.\n",
                           iTransition, iPrevState, procState->iState);
               }
               break;
           case PROC_RUNNING:
               if (checkTransProc(iPrevState, iTransition) != PROC_RUNNING) 
			   {
                   cRetVal = 0;
                   TSTLOG_appendLog(pHandle,
                           "\nTransition %i from state %i to %i invalid.\n",
                           iTransition, iPrevState, procState->iState);
               }
               break;
           default:
               cRetVal = 0;
               TSTLOG_appendLog(pHandle, "State %i doesn't exist!\n",
                                procState->iState);
               szErrorMsg = "Landed in unknown state!\n";
               break;
           }
       }

	} // end of while loop
    } // end of if stmt for cRetVal

    /* If Error, detach Processor */
    if (cRetVal == 0) 
	{
        DSPProcessor_Detach(hProcessor);
    }

    OsSpecificMemFree(pArgs);
    OsSpecificMemFree(pAttrIn);
    OsSpecificMemFree(procState);
    OsSpecificMemFree(aNodeTab);
    OsSpecificMemFree(puNumNodes);
    OsSpecificMemFree(puAllocated);

    if (cRetVal == 0) 
	{
        /* Report in LOG module */
        if (bUseLogFile) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle, "Error: %s\n", szErrorMsg);
        }
        TST_verbose("FAIL\n");
    }
    else 
	{
        /* Report in LOG module */
        if (bUseLogFile) 
		{
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLogFile) 
	{
        TSTLOG_closeLog(pHandle);
    }

    return 0; /* dummy return - swamy */
}

/*
 *  ======== checkTransProc ========
 *  This function is specific to the DSPProcessor Module.
 *  It is configured to calculate, given a present state and a transition,
 *  which state it should end up in.  This is used to check whether
 *  or not a given transition performed correctly.
 */
INT checkTransProc(const INT iPrevState, const INT iTransition)
{
    INT iGoalState;

    switch(iTransition) 
	{
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
        case 6:
        case 9:
            iGoalState = iPrevState;
            break;
        case 7:
            iGoalState = PROC_LOADED;
            break;
        case 8:
            if (iPrevState == PROC_LOADED) 
			{
                iGoalState = PROC_RUNNING;
            }
            else 
			{
                iGoalState = iPrevState;
            }
            break;
        default:
            iGoalState = 0;
            break;
   }
   return iGoalState;
}

