/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    dspstreamst.c
 *
 *   Path    $(TESTSUITEROOT)\Behaviour\DSPStreamState
 * 
 *   Desc:
 *        This module tests the state of a stream following various
 *        DSPStream API function calls.  This test allows various
 *        APIs which alter the state of the Node.  A Node
 *        is allocated and various function calls that alter the state are
 *        issued by the user in a given sequence to verify the correct
 *        board transitions.  If the user supplied sequence is
 *        incorrect, then one function call may result in a failure.
 *        If DSPProcessor_Attach is not called as the first function,
 *        the sequence will fail.
 *
 *  Usage:
 *        dspstreamst [-v] [-l "log path/filename"]
 *                       [-x "valid executable" ]
 *                       [-t "state transitions"]
 *
 *        -v: verbose mode
 *        -l: use output log and location
 *        -x: name of valid DSP executable
 *        -t: input of state transition sequence
 *
 *  API Codes to use to specify sequence as input:
 *
 *         1: DSPStream_Open
 *         2: DSPStream_GetInfo
 *         3: DSPStream_AllocateBuffers
 *         4: DSPStream_FreeBuffers
 * 	       5: DSPStream_PrepareBuffer
 *         6: DSPStream_UnprepareBuffer
 *         7: DSPStream_Issue
 *         8: DSPStream_Reclaim
 *         9: DSPStream_Select
 *         a: DSPStream_RegisterNotify
 *         b: DSPStream_Idle
 *         c: DSPStream_Close
 *         d: DSPStream_Issue - with 0 bytes - to goto DONE state
 *         e: Special case to check for Pending state - Pause the Node
 *
 *  Example:
 *  dspstreamst -v -l "output.txt" -t "1 3 7 8 b c"
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL 03, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>

#define BUFSIZE             64

#define DEFNAMESIZE         80
#define DEFAULTDSPUNIT 	    0
#define DSPEXEC             "dataxfr_nortdxidl.dof"

#define DEFSTATES           "1 2"
#define SEPARATORS          " , \t\n"

INT checkTransStream(const INT iPrevState, const INT iTransition);

/*
 *  ======== dspstreamst_main ========
 *
 */
INT dspstreamst_main (INT argc, CHAR * argv[])
{

    INT                     opt, i;
    INT                     cRetVal = 1;
    INT                     iPrevState;
    INT                     iTransition = 0;
    INT                     iArgc = 1;
    BOOL                    bUseLog=FALSE;
    BOOL                    bStrmOpened=FALSE;
    BOOL                    bStrmIssued=FALSE;
    FILE                    *pHandle = NULL;
    CHAR                    szLogName[DEFNAMESIZE];
    CHAR                    *szErrorMsg = "";
    CONST CHAR              *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    CHAR                    *szProcStates;
    CHAR                    szDefaultStates[DEFNAMESIZE];
    CONST CHAR              *aEnvp[DEFNAMESIZE] = {NULL};
    DWORD                   dwArg;
    UINT                    uIndex = 0;
    UINT                    *pMask;
    ULONG                   pBytes = 0;
    DSP_HPROCESSOR          hProcessor;
    DSP_HNODE               hNode;
    struct DSP_NODEATTRIN          nodeAttrin;
    DSP_HSTREAM             hStream;
    DSP_HSTREAM             *aStreamTab;
    struct DSP_STRMATTR            attrs;
    struct DSP_STREAMATTRIN        strmAttrs;
    struct DSP_STREAMINFO          *pStreamInfo;
    DSP_STATUS              status = DSP_SOK;
    DSP_STATUS              exitStatus;
    struct DSP_UUID                gidNodeID;
    struct DSP_PROCESSORATTRIN     *pAttrIn;
    struct DSP_CBDATA              *cbData;
    struct DSP_MSG                 *pMessage;
    CHAR                    *pBuffer[1];
    struct DSP_NOTIFICATION*       hNotification;
    BYTE                    argsBuf[32 * sizeof(ULONG)];
    CHAR                    *pstrList[50];
    CHAR                    *pszUuid;
    INT                     cListsize = 0;
    INT                     count = 0;
    INT                     chkTranRetVal;
    DWORD                   dwSize;
    BOOL					fLoadDsp = FALSE;



    hNotification = OsSpecificMemAlloc(sizeof(struct DSP_NOTIFICATION*));

    cbData = (struct DSP_CBDATA *)argsBuf;
    cbData->cbData = 32;
    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;
    pStreamInfo = OsSpecificMemAlloc(sizeof(struct DSP_STREAMINFO));
    pMessage = OsSpecificMemAlloc(sizeof(struct DSP_MSG));
    aStreamTab = OsSpecificMemAlloc(sizeof(DSP_HSTREAM));
    pMask = OsSpecificMemAlloc(sizeof(UINT));

    nodeAttrin.uTimeout = 1000;
    nodeAttrin.iPriority = 1;

#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif

    attrs.uBufsize = 64;
    attrs.uNumBufs = 1;
    attrs.uAlignment = 0;
    attrs.uTimeout = DSP_FOREVER;
    attrs.lMode = STRMMODE_PROCCOPY;
    attrs.uSegid = 0;

    strmAttrs.cbStruct = sizeof(struct DSP_STREAMATTRIN);
    strmAttrs.uTimeout = 5000;
    strmAttrs.uAlignment = 0;
    strmAttrs.uNumBufs = 1;
    strmAttrs.lMode = STRMMODE_PROCCOPY;
    strmAttrs.uSegment = 0;

    /* Set default set of state transitions */
    strncpy(szDefaultStates, (char *)DEFSTATES, DEFNAMESIZE);
    szProcStates = szDefaultStates;

	optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:t:")) != EOF) 
    {
        switch (opt) 
    	{
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            pHandle = TSTLOG_openLog(szLogName);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            fLoadDsp = TRUE;
            break;
        case 't':
            /* input state transitions */
            szProcStates = (char *) optarg;
            if (bUseLog) 
    	    {
                TSTLOG_appendLog(pHandle, "Using board states: \"%s\"\n",
                                 szProcStates);
            }
            TST_verbose("Using board states: \"%s\"\n", szProcStates);
            break;
        default:
            TST_verbose(("Invalid option: %c\n"), opt);
            cRetVal = 0;
            szErrorMsg = "Invalid option specified.";
            break;
        }
    }

    if (cRetVal > 0) 
    {
        /* Attach to processor */
        struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    index=0;
        INT                     procId=0;
    
        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
            if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
                TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                procId = index;
                status = DSP_SOK;
                break;
            }
            index++;
        }

        /* Attach to an available DSP (in this case, the 1st DSP). */
        if (DSP_SUCCEEDED(status)) {
        
            status = DSPProcessor_Attach(procId, pAttrIn, &hProcessor);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
            }
        }
        else {
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
        }

	    if(fLoadDsp)
	    {
		DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, (CONST CHAR **)aEnvp);
		DSPProcessor_Start(hProcessor);
		}
        pszUuid = "F95CA072_EE7D_11D5_A8DD_00B0D055F6D1";  // dataxfr
        UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

	    /* Allocate Node */
		status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData, &nodeAttrin,
                  &hNode);

		status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0, &attrs);

		status =DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0, &attrs);

		status = DSPNode_Create(hNode);

		status = DSPNode_Run(hNode);

		/* Process API list */
		/*pstrList[cListsize++] = strtok(szProcStates, SEPARATORS);
		while ((pstrList[cListsize++] = strtok(NULL, SEPARATORS)) != NULL);*/

		/* Adjust for NULL element */
		cListsize--;

        while (cRetVal > 0 && count < cListsize) 
		{
            switch(pstrList[count++][0]) 
		    {
            case '1':
                /* DSPStream_Open */
                iTransition = 1;

                status = DSPStream_Open(hNode, DSP_TONODE, uIndex++,
                    &strmAttrs, &hStream);

				bStrmOpened = TRUE;
                /* Get Initial State -- okay to do here for Open */
                status = DSPStream_GetInfo(hStream, pStreamInfo, sizeof(struct DSP_STREAMINFO));

                szErrorMsg = "Error in DSPStream_Open()";
                break;
            
            case '2':
				/* DSPStream_GetInfo */
                iTransition = 2;

                status = DSPStream_GetInfo(hStream, pStreamInfo, sizeof(struct DSP_STREAMINFO));

                szErrorMsg = "Error in DSPStream_GetInfo()";
                break;
            
            case '3':
				/* DSPStream_AllocateBuffers */
                iTransition = 3;

                status = DSPStream_AllocateBuffers(hStream, BUFSIZE, (BYTE **)pBuffer, 1);

                szErrorMsg = "Error in DSPStream_AllocateBuffers()";
                break;
            
            case '4':
				/* DSPStream_FreeBuffers */
                iTransition = 4;

                status = DSPStream_FreeBuffers(hStream, (BYTE **)pBuffer, 1);

                szErrorMsg = "Error in DSPStream_FreeBuffers()";
                break;
			
            case '5':
				/* DSPStream_PrepareBuffer */
				iTransition = 5;

				status = DSPStream_PrepareBuffer(hStream, BUFSIZE, pBuffer[0]);

				szErrorMsg = "Error in DSPStream_PrepareBuffer()";
				break;
			
            case '6':
				/* DSPStream_PrepareBuffer */
				iTransition = 6;

				status = DSPStream_UnprepareBuffer(hStream, BUFSIZE, pBuffer[0]);

				szErrorMsg = "Error in DSPStream_UnprepareBuffer()";

				break;
			
            case '7':
				/* DSPStream_Issue */
				iTransition = 7;

				pMessage->dwCmd = 2;
				status = DSPNode_PutMessage(hNode, pMessage, 500);

				for (i = 0; i < (INT) BUFSIZE; i++) 
				{
					pBuffer[0][i] = (BYTE) 'a';
				}

				dwArg = 0x0;
				status = DSPStream_Issue(hStream, pBuffer[0], BUFSIZE, BUFSIZE, dwArg);
				bStrmIssued = TRUE;

				szErrorMsg = "Error in DSPStream_Issue()";
				break;
            
            case '8':
				/* DSPStream_Reclaim */
                iTransition = 8;

                status = DSPStream_Reclaim(hStream, (BYTE **)&pBuffer[0], &pBytes, &dwSize, &dwArg);
				bStrmIssued = FALSE;

                szErrorMsg = "Error in DSPStream_Reclaim()";
                break;
            
            case '9':
				/* DSPStream_Select */
                iTransition = 9;

                aStreamTab[0] = hStream;

                status = DSPStream_Select(aStreamTab, 1, pMask, 50);

                szErrorMsg = "Error in DSPStream_Select()";
                break;
            
            case 'a':
				/* DSPStream_RegisterNotify */
                iTransition = 10;

                status = DSPStream_RegisterNotify(hStream, DSP_STREAMIOCOMPLETION,
                           DSP_SIGNALEVENT, hNotification);

                szErrorMsg = "Error in DSPStream_RegisterNotify()";
                break;
            
            case 'b':
				/* DSPStream_Idle */
                iTransition = 11;

                status = DSPStream_Idle(hStream, TRUE);

                szErrorMsg = "Error in DSPStream_Idle()";
                break;
            
            case 'c':
				/* DSPStream_Close */
                iTransition = 12;

                status = DSPStream_Close(hStream);
				bStrmOpened = FALSE;

                szErrorMsg = "Error in DSPStream_Close()";
                break;
			
            case 'd':
				/* DSPStream_Issue with 0 bytes - goto DONE state*/
				iTransition = 13;

				pMessage->dwCmd = 2;
				status = DSPNode_PutMessage(hNode, pMessage, 500);
				
				for (i = 0; i < (INT) BUFSIZE; i++) 
				{
					pBuffer[0][i] = (BYTE) 'a';
				}

				dwArg = 0x0;
				status = DSPStream_Issue(hStream, pBuffer[0], 0, BUFSIZE, dwArg);
				bStrmIssued = TRUE;

				szErrorMsg = "Error in DSPStream_Issue() for DONE state";
				break;

            case 'e':
				/* DSPStream_Issue with Node Pause etc. To check Pending state*/
				break;
            default: TST_verbose("Invalid API call: %c\n", pstrList[cListsize-1][0]);
                break;
            } /* End of switch pstrList */

			if (!DSP_SUCCEEDED(status)) 
			{
			   cRetVal = 0;
			   break;
			}

			/* Get state and check for correctness, unless detached */
			if (iTransition != 12) 
			{
			  iPrevState = pStreamInfo->ssStreamState;
			  status=DSPStream_GetInfo(hStream, pStreamInfo, sizeof(struct DSP_STREAMINFO));
			}
			else 
			{
			  continue;
			}

			if (!DSP_SUCCEEDED(status)) 
			{
			   cRetVal = 0;
			   szErrorMsg = "Error in DSPStream_GetInfo()\n";
			   break;
			}
			else 
			{
				/* Check to see if we are in the correct state */
				switch(pStreamInfo->ssStreamState) 
				{
				case STREAM_IDLE:
				    if (checkTransStream(iPrevState, iTransition) != STREAM_IDLE) 
					{
					   cRetVal = 0;
					   TSTLOG_appendLog(pHandle,
								"\nTransition %i from state %i to %i invalid.\n",
								iTransition, iPrevState,
								pStreamInfo->ssStreamState);

					   break;
					}
				    break;
				case STREAM_PENDING:
					chkTranRetVal = checkTransStream(iPrevState, iTransition);
				    if (!((chkTranRetVal == STREAM_PENDING) || (chkTranRetVal == STREAM_READY)))
					{
					    cRetVal = 0;
					    TSTLOG_appendLog(pHandle,
								"\nTransition %i from state %i to %i invalid.\n",
								iTransition, iPrevState,
								pStreamInfo->ssStreamState);

					    break;
					}
					break;
				case STREAM_READY:
					if (checkTransStream(iPrevState, iTransition) != STREAM_READY) 
					{
						cRetVal = 0;
						TSTLOG_appendLog(pHandle,
								"\nTransition %i from state %i to %i invalid.\n",
								iTransition, iPrevState,
								pStreamInfo->ssStreamState);

					    break;
					}
				    break;
				case STREAM_DONE:
				    if (checkTransStream(iPrevState, iTransition) != STREAM_DONE) 
					{
					    cRetVal = 0;
					    TSTLOG_appendLog(pHandle,
								"\nTransition %i from state %i to %i invalid.\n",
								iTransition, iPrevState,
								pStreamInfo->ssStreamState);

					    break;
					}
				    break;
				default:
					cRetVal = 0;
					TSTLOG_appendLog(pHandle, "State %i doesn't exist!\n",
									pStreamInfo->ssStreamState);
					szErrorMsg = "Landed in unknown state!\n";
					break;
				} /* End of Switch stmt */
			} /* End of else for DSP_SUCCEEDED(status) */
        } /* End of while cRetVal */
    } /* End of if cRetVal > 0 */

	if (!DSP_SUCCEEDED(status)) 
	{
		cRetVal = 0;
		TSTLOG_appendLog(pHandle, "ERROR status = 0x%x\n",status);
		szErrorMsg = "General Error - FAIL\n";
	}

    if (bStrmIssued) 
    {
        DSPStream_Reclaim(hStream, (BYTE **)&pBuffer[0], &pBytes, &dwSize, &dwArg);
    }

    pMessage->dwCmd = 1;
    DSPNode_PutMessage(hNode, pMessage, 500);
    
    if (bStrmOpened) 
    {
        DSPStream_Close(hStream);
    }

    status = DSPNode_Terminate(hNode, &exitStatus);
    status = DSPNode_Delete(hNode);
    DSPProcessor_Detach(hProcessor);

    OsSpecificMemFree(pAttrIn);
    OsSpecificMemFree(pMask);
    OsSpecificMemFree(aStreamTab);
    OsSpecificMemFree(pStreamInfo);
    OsSpecificMemFree(pMessage);
    OsSpecificMemFree(hNotification);

    if (cRetVal == 0) 
    {
        /* Report in LOG module */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle, "Error: %s\n", szErrorMsg);
        }
        TST_verbose("FAIL\n");
    }
    else 
    {
        /* Report in LOG module */
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) 
    {
        TSTLOG_closeLog(pHandle);
    }

    return 0; /* dummy return - swamy */
}

/*
 *  ======== checkTransStream ========
 *  This function is specific to the DSPStream Module.
 *  It is configured to calculate, given a present state and a transition,
 *  which state it should end up in.  This is used to check whether
 *  or not a given transition performed correctly.
 */
INT checkTransStream(const INT iPrevState, const INT iTransition)
{
    INT iGoalState = 0;

    switch(iTransition) 
    {
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
        case 6:
		case 9:
		case 10:
		case 12:
            iGoalState = iPrevState;
            break;
        case 7:
            if (iPrevState == STREAM_IDLE) 
			{
            /* In READY state AFTER STREAM_PENDING state */
                iGoalState = (STREAM_READY);
            }
            else 
			{
                iGoalState = iPrevState;
            }
            break;
        case 8:
            if (iPrevState == STREAM_READY) 
			{
                iGoalState = (STREAM_IDLE);
            }
            else 
			{
                iGoalState = iPrevState;
			}
            break;
        case 11:
			if (iPrevState == STREAM_DONE) 
			{
				iGoalState = 0;
            }
            iGoalState = STREAM_IDLE;
            break;
		case 13:
			if (iPrevState == STREAM_IDLE)
				iGoalState = STREAM_DONE;
			break;
        default:
            iGoalState = 0;
            break;
   }
   return iGoalState;
}

