/*
*  Copyright 2004 by Texas Instruments Incorporated.
*  All rights reserved. Property of Texas Instruments Incorporated.
*  Restricted rights to use, duplicate or disclose this code are
*  granted through contract.
*
*  @(#) DSP/BIOS Bridge 
*/
/*  ============================================================================
*   File    QueueData.c
*
*   Path    $(TESTSUITEROOT)\Behaviour\Node
* 
*   Desc:   This test case Queues multiple data buffers
*           for Issue and Reclaims them.
*
*  Usage:
*        MULTHRDDATAQ [-v] [-l "log path/filename"] 
*                      [-x "valid executable"] ..
*
*        -v: verbose mode
*        -l: use log and location
*        -x: DSP Image to be used
*        -n: Number of Transfers for each Delay
*        -s: Buffer Size
*        -q: Number of buffers to be queued
*
*   ---------------------------------------------------------------------------
*   Rev     1.0.0
* 
*   This computer program is copyright to Texas Instruments Incorporated.
*   The program may not be used without the written permission
*   of Texas Instruments Incorporated. or against the terms and conditions
*   stipulated in the agreement under which this program has been
*   supplied.
*
*   (c) Texas Instruments Incorporated 2003
*
*   ============================================================================
*   Revision History
*
*   AUG 20, 2002    REF=ORG             Swamy MC
*      Original Version.
*   ============================================================================
*/


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>

#include "QueueData.h"
#define DEFAULTDSPUNIT   0
#define DEFNAMESIZE      100
#define NUMOFCOMPARGS      3    /* Compulsory args: t,n,s,q */
#define PASS               0
#define FAIL              -1
#define MAXNUMOFBUFFERS    4    /* Depends on the CDB file for the DSP executable */

#define SHORT short

#define DSPEXEC        "inout_nortdxidl.dof"

#define CLOSELOG    if (bUseLog) { \
    TSTLOG_closeLog(pHandle); \
                    } 


/*
* Function to test the Data Queuing behaviour of the Bridge
*/
static INT queueDataFunc(FuncArgs *args)
{
    
    struct DSP_STRMATTR            attrs;
    BYTE                    **arrOutBuf;
    BYTE                    **arrInBuf;
    struct DSP_NODEATTRIN          nodeAttrin;
    DSP_HNODE               hNode = NULL;
    DSP_HSTREAM             hOutStream;
    DSP_HSTREAM             hInStream;
    CHAR                    *pszUuid;
    struct DSP_UUID                gidNodeID;
    DSP_STATUS              status;
    INT                     i,j;
    DSP_STATUS              exitStatus = DSP_SOK;
    INT                     iQueue;
    ULONG                   dwBufSize1;
    ULONG                   dwBufSize2;
    DWORD                   dwArg = 0;
    
    /* Processor related information */
    DSP_HPROCESSOR          hProcessor;
    
    /* Test case specific information */
    INT                     iNumTransfers;
    INT                     iBuffSize;
    
    /* Logging related information */
    FILE                    *pHandle;
    BOOL                    bUseLog=FALSE;
    BOOL                    bVerbose=FALSE;
    
    /* Copy the contents of passed args to local vars */
    /* This strategy can be changed if the variables are just
    used/read and not modified. (Swamy) */
    hProcessor = args->hProcessor;
    iNumTransfers = args->iNumTransfers;
    iBuffSize = args->iBuffSize;
    pHandle = args->pHandle;
    bUseLog = args->bUseLog;
    bVerbose = args->bVerbose;
    iQueue = args->iQueue;
    
    /* Initialising attributes */
    attrs.uSegid = 0;
    attrs.uBufsize = iBuffSize;
    attrs.lMode = STRMMODE_PROCCOPY;
    attrs.uNumBufs = iQueue;
    attrs.uAlignment = 0;
    attrs.uTimeout = 5000;
    
    nodeAttrin.iPriority = 5 ;
    nodeAttrin.uTimeout = 10000 ;
    
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif

    arrOutBuf = OsSpecificMemAlloc(sizeof(BYTE *) * iQueue);
    arrInBuf = OsSpecificMemAlloc(sizeof(BYTE *) * iQueue);
    
    
    pszUuid = "F95CA076_EE7D_11D5_A8DD_00B0D055F6D1" ; // taken from inout_nortdx_int.cdb file
    UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);
    
    /* Allocate Node */
    status = DSPNode_Allocate(hProcessor, &gidNodeID, NULL, &nodeAttrin,
        &hNode);
    
    if (DSP_FAILED(status)) 
    {
        /* Report Error */
        if (bUseLog) 
        {
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Allocate(). status = 0x%x\n",status);
        }
        return FAIL;
    }
    
    status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0,
        &attrs);
    
    if (DSP_FAILED(status)) 
    {
        /* Report Error */
        if (bUseLog) 
        {
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect. status = 0x%x\n",status);
        }
        return FAIL;
    }
    
    status = DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0,
        &attrs);
    
    if (DSP_FAILED(status)) 
    {
        /* Report Error */
        if (bUseLog) 
        {
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect. status = 0x%x\n",status);
        }
        return FAIL;
    }
    
    
    /* Create Node on DSP */
    status = DSPNode_Create(hNode);
    
    if (DSP_FAILED(status)) 
    {
        /* Report Error */
        if (bUseLog) 
        {
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Create. status = 0x%x\n",status);
        }
        return FAIL;
    }
    
    status = DSPNode_Run(hNode);
    
    if (DSP_FAILED(status)) 
    {
        /* Report Error */
        if (bUseLog) 
        {
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Run. status = 0x%x\n",status);
        }
        return FAIL;
    }
    
    status = DSPStream_Open(hNode, DSP_TONODE, 0, NULL, &hOutStream);
    
    if (DSP_FAILED(status)) 
    {
        /* Report Error */
        if (bUseLog) 
        {
            TSTLOG_appendLog(pHandle,"Error in DSPStream_Open. status = 0x%x\n",status);
        }
        return FAIL;
    }
    
    status = DSPStream_Open(hNode, DSP_FROMNODE, 0, NULL, &hInStream);
    
    if (DSP_FAILED(status)) 
    {
        /* Report Error */
        if (bUseLog) 
        {
            TSTLOG_appendLog(pHandle,"Error in DSPStraem_Open. status = 0x%x\n",status);
        }
        return FAIL;
    }
    
    status = DSPStream_AllocateBuffers(hOutStream, iBuffSize, arrOutBuf, iQueue);
    
    if (DSP_FAILED(status)) 
    {
        /* Report Error */
        if (bUseLog) 
        {
            TSTLOG_appendLog(pHandle,"Error in DSPStream_AllocateBuffers. status = 0x%x\n",status);
        }
        return FAIL;
    }
    
    status = DSPStream_AllocateBuffers(hInStream, iBuffSize, arrInBuf, iQueue);
    
    if (DSP_FAILED(status)) 
    {
        /* Report Error */
        TST_verbose("FAIL\n");
        
        if (bUseLog) 
        {
            TSTLOG_appendLog(pHandle,"Error in DSPStream_AllocateBuffers. status = 0x%x\n",status);
        }
        return FAIL;
    }
    
    /* fill All Output buffers */
    for (i = 0; i < iQueue; i++)
    {
        for (j = 0; j < iBuffSize; j++) 
        {
            arrOutBuf[i][j] = (BYTE) 'a';
        }
    }
    /*  Data transfer for number of iterations .. specified by the
    *  user as parameters to the test ... set in the constructor
    */
    for (i = 0 ; i < iNumTransfers ; i++)
    {
        for (j = 0; j < iQueue; j++)
        {
            /* Issue Data */
            status = DSPStream_Issue(hOutStream, arrOutBuf[j], iBuffSize, iBuffSize, 0);
            
            if (DSP_FAILED(status)) 
            {
                /* Report Error */
                if (bUseLog) 
                {
                    TSTLOG_appendLog(pHandle,"Error in DSPStraem_Issue. status = 0x%x\n",status);
                }
                return FAIL;
            }
        } // end of for "j"
        
        for (j = 0; j < iQueue; j++)
        {
            status = DSPStream_Reclaim(hOutStream, &arrOutBuf[j], &dwBufSize1, &dwBufSize2, &dwArg);
            if (DSP_FAILED(status)) 
            {
                /* Report Error */
                if (bUseLog) 
                {
                    TSTLOG_appendLog(pHandle,"Error in DSPStraem_Reclaim. status = 0x%x\n",status);
                }
                return FAIL;
            }
        }
        for (j = 0; j < iQueue; j++)
        {
            /* Issue Data */
            status = DSPStream_Issue(hInStream, arrInBuf[j], iBuffSize, iBuffSize, 0);
            
            if (DSP_FAILED(status)) 
            {
                /* Report Error */
                if (bUseLog) 
                {
                    TSTLOG_appendLog(pHandle,"Error in DSPStraem_Issue. status = 0x%x\n",status);
                }
                return FAIL;
            }
        } // end of for "j"
        
        for (j = 0; j < iQueue; j++)
        {
            status = DSPStream_Reclaim(hInStream, (BYTE **)&arrInBuf[j], 
                &dwBufSize1, &dwBufSize2, &dwArg);
            
            if (DSP_FAILED(status)) 
            {
                /* Report Error */
                if (bUseLog) 
                {
                    TSTLOG_appendLog(pHandle,"Error in DSPStraem_Reclaim. status = 0x%x\n",status);
                }
                return FAIL;
            }
        }
    } // End of For loop - iNumTransfers
    
    
    status = DSPStream_FreeBuffers(hOutStream, arrOutBuf, iQueue);
    status = DSPStream_FreeBuffers(hInStream, arrInBuf, iQueue);
    
    status = DSPStream_Close(hOutStream);
    status = DSPStream_Close(hInStream);
    status = DSPNode_Terminate(hNode, &exitStatus);
    status = DSPNode_Delete(hNode);
    
    OsSpecificMemFree(arrOutBuf);
    OsSpecificMemFree(arrInBuf);
    
    return 0;
}


/*
*  ======== queueData_BEHAVIOUR ========
*
*/
INT queueData_BEHAVIOUR(INT argc, CHAR * argv[])
{
    BOOL                    bUseLog=FALSE;
    BOOL                    bVerbose=FALSE;
    FILE                    *pHandle = NULL;
    INT                     iNumTransfers = 0;
    INT                     iBuffSize = 0;
    INT                     iQueue = 0;
    CHAR                    szLogName[DEFNAMESIZE];
    INT                     iMandArgCount = NUMOFCOMPARGS;
    DSP_STATUS              status;
    FuncArgs                funcArgs;
    
    struct DSP_PROCESSORATTRIN     *pAttrIn;
    DSP_HPROCESSOR          hProcessor;
    CHAR                    *aArgv[DEFNAMESIZE] = { DSPEXEC, "" };
    CHAR                    *aEnvp[DEFNAMESIZE] = {NULL};
    INT                     iArgc = 1;
    INT                     opt;
    INT                     retVal = PASS;
    BOOL                    fLoadDsp = FALSE;
    
    
    /* Check Number of Arguments */
    /* Process arguments and Store the values */
    /* Load the Image on DSP */
    /* Allocate Buffers for Transfer of Data */
    /* Set the Properties, Allocate and Create the Node on DSP */
    /* Open the Streams */
    /* Run the task */
    /* Queue the Data Buffers and check for successful Data Transfer */
    /* End !! */
    /* Process arguments and Store the values */
    optind = 0;
    opt = getopt_test(argc, (char *const)argv, "+vl:x:n:s:q:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            pHandle = TSTLOG_openLog(szLogName);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            fLoadDsp = TRUE;
            break;
        case 'n':
            /* Number of Transfers */
            iNumTransfers = CSL_Atoi(optarg);
            iMandArgCount--;
            break;
        case 's':
            /* Buffer Size */
            iBuffSize = CSL_Atoi(optarg);
            iMandArgCount--;
            break;
        case 'q':
            /* Number of Messages to be Queued */
            iQueue = CSL_Atoi(optarg);
            iMandArgCount--;
            if (iQueue > MAXNUMOFBUFFERS)
                iQueue = MAXNUMOFBUFFERS;
            break;
        default:
            TST_reportError(("Invalid option: %c\n"), opt);
            
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                    "Invalid option specified.\n");
            }
            break;
        }
        opt = getopt_test(argc, (char *const)argv, "+vl:x:n:s:q:");
    }
    
    /* If all the Mandatory args are not present, abort */
    if (iMandArgCount != 0)
    {
        // Error
        TST_verbose("Number of Mandatory Arguments is incorrect.\n");
        TST_verbose("Some args may be missing\n");
        TSTLOG_appendLog(pHandle, "FAIL : ");
        TSTLOG_appendLog(pHandle,"Number of Mandatory Arguments is incorrect.\n");
        CLOSELOG;
        return 0;
    }
    
    TST_verbose("DSP Executable: %s\n", aArgv[0]);
    
    
    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    if (pAttrIn == NULL)
    {
        // Error
        TST_verbose("Malloc for pAttrIn FAILED !!\n");
        TSTLOG_appendLog(pHandle,"Malloc for pAttrIn FAILED !!\n");
        CLOSELOG;
        return 0;
    }
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;
    
    /* Attach to processor */
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId=0;
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }
    
    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, pAttrIn, &hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    }
    
    if (DSP_FAILED(status)) 
    {
        /* Report Error */
        TST_verbose("FAIL\n");
        
        if (bUseLog) 
        {
            TSTLOG_appendLog(pHandle,"Error in DSPProcessor_Attach. status = 0x%x\n",status);
        }
        CLOSELOG;
        return 0;
    }

    if(fLoadDsp)
    {
        /* Load default program */
        status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, 
            (CONST CHAR **)aEnvp);
        if (DSP_FAILED(status)) 
        {
            /* Report Error */
            if (bUseLog) 
            {
                TSTLOG_appendLog(pHandle, "Error in DSPProcessor_Load. status = 0x%x\n",status);
            }
            CLOSELOG;
            return 0;
        }
        
        status = DSPProcessor_Start(hProcessor);
        if (DSP_FAILED(status)) 
        {
            /* Report Error */
            if (bUseLog) 
            {
                TSTLOG_appendLog(pHandle, "Error in DSPProcessor_Start. status = 0x%x\n",status);
            }
            CLOSELOG;
            return 0;
        }
    }
    
    /* Fill the Args which are to be passed on to the Function */
    funcArgs.hProcessor = hProcessor;
    funcArgs.iNumTransfers = iNumTransfers;
    funcArgs.iBuffSize = iBuffSize;
    funcArgs.pHandle = pHandle;
    funcArgs.bUseLog = bUseLog;
    funcArgs.bVerbose = bVerbose;
    funcArgs.iQueue = iQueue;
    
    status = queueDataFunc(&funcArgs);
    if (DSP_FAILED(status)) 
    {
        /* Report Error */
        if (bUseLog) 
        {
            TSTLOG_appendLog(pHandle, "Error in queueDataFunc. status = 0x%x\n",status);
        }
        CLOSELOG;
        return 0;
    }
    
    
    DSPProcessor_Detach(hProcessor);
    OsSpecificMemFree(pAttrIn);
    
    if (retVal == PASS) 
    {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }
    
    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }
    
    return 0;
}

