/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    MultiThreadMsgQueue.c
 *
 *   Path    $(TESTSUITEROOT)\Stress\MultiThread\MultiThreadMsgQueue.c
 * 
 *   Desc:
 *           This test case tests the Queuing mechanism
 *           of Msg Transfer by sending multiple messages to Msg Queue.
 *
 *  Usage:
 *        MULTHRDMSGQ   [-v] [-l "log path/filename"] 
 *                      [-x "valid executable"] ..
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: DSP Image to be used
 *        -t: Number of Threads to be created
 *        -n: Number of Transfers for each Delay
 *        -s: Buffer Size
 *        -q: Number of messages to be queued
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   AUG 20, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <dbapi.h>
#include <getopt.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>
#include "MultiThread.h"

#define DEFNAMESIZE      100
#define NUMOFCOMPARGS      4	/* Compulsory args: t,n,s,q */
#define MAXNUMOFTHREADS    4    /* This number taken from Symbian Test Suite */
#define PASS               0
#define FAIL              -1
#define DEFAULTDSPUNIT     0
#define SHORT short

#define DSPEXEC        "msg_nortdxidl.dof"


/* Function to exit gracefully */
static INT graceful_exit()
{
	printf("FATAL ERROR: Exiting the Test Case !\n");
	exit(0);
}

/* Thread function for Multi thread data transfer test case */
/*
 * Note: Address of args is being passed from the main thread,		*
 * dont alter the contents as this will be used/needed by all the	*
 * threads created. Copy the contents to local variables and use	*
 */
static INT threadFxn(MultiThreadArgs *targs)
{

	BYTE					**arrBuf;
    	struct DSP_NODEATTRIN			nodeAttrin;
    	DSP_HNODE				hNode = NULL;
	CHAR					*pszUuid;
	struct DSP_UUID				gidNodeID;
    
	DSP_STATUS				status;
	INT						i,j;
	DSP_STATUS				exitStatus = DSP_SOK;
	struct DSP_MSG                 msg;
	UINT                    uTimeout = 10000;

	/* Processor related information */
	DSP_HPROCESSOR			hProcessor;

	/* Test case specific information */
	INT						iNumTransfers;
	INT						iBuffSize;
	INT                     iQueue;

	/* Logging related information */
	FILE					*pHandle;
	BOOL					bUseLog=FALSE;
	BOOL					bVerbose=FALSE;


	/* Copy the contents of passed args to local vars */
	/* This strategy can be changed if the variables are just
	   used/read and not modified. */
	hProcessor = targs->hProcessor;
	iNumTransfers = targs->iNumTransfers;
	iBuffSize = targs->iBuffSize;
	pHandle = targs->pHandle;
	bUseLog = targs->bUseLog;
	bVerbose = targs->bVerbose;
	iQueue = targs->iQueue;

	nodeAttrin.cbStruct = sizeof(struct DSP_NODEATTRIN);
    nodeAttrin.uTimeout = 1000;
    nodeAttrin.iPriority = 1;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif



    /* Initialising attributes */

	arrBuf = OsSpecificMemAlloc(sizeof(BYTE *));


	pszUuid = "F95CA077_EE7D_11D5_A8DD_00B0D055F6D1";  // msg_nortdx_int.cdb
    UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

    /* Allocate Node */
    status = DSPNode_Allocate(hProcessor, &gidNodeID, NULL, &nodeAttrin,
                   &hNode);
	TST_verbose("DSPNode_Allocate ret status: %x\n",status);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Allocate().\n");
        }
        return FAIL;
    }


    /* Create Node on DSP */
    status = DSPNode_Create(hNode);
	TST_verbose("DSPNode_Create ret status: %x\n",status);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Create()\n");
        }
        return FAIL;
    }

    status = DSPNode_Run(hNode);
	TST_verbose("DSPNode_Run ret status: %x\n",status);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Run()\n");
        }
        return FAIL;
    }

	/* Do the Msg Transfer */
    /*  Msg transfer for number of iterations .. specified by the
     *  user as parameters to the test ... set in the constructor
	 */
    for (i = 0 ; i < iNumTransfers ; i++)
	{

		for(j=0; j<iQueue; j++)
		{
			msg.dwCmd = RMS_USER + 3;
			msg.dwArg1 = j;  // does not matter
			msg.dwArg2 = j;  // does not matter

			status = DSPNode_PutMessage(hNode, &msg, uTimeout);
			TST_verbose("DSPNode_PutMessage ret status: %x\n",status);
			if (!DSP_SUCCEEDED(status))
			{
				TSTLOG_appendLog(pHandle, 
					"DSPNode_PutMessage failed. Status = 0x%x\n", status);
				return FAIL;
			}
		}

		for(j=0; j<iQueue; j++)
		{
			status = DSPNode_GetMessage(hNode, &msg, uTimeout);
			if (DSP_SUCCEEDED(status)) 
			{
				TST_verbose("Got message: 0x%x 0x%x 0x%x\n", msg.dwCmd,
					msg.dwArg1, msg.dwArg2);
				TSTLOG_appendLog(pHandle, "Got message: 0x%x 0x%x 0x%x\n", msg.dwCmd,
					msg.dwArg1, msg.dwArg2);
			}
			else 
			{
				TST_verbose("DSPNode_GetMessage failed. Status = 0x%x\n", status);
				TSTLOG_appendLog(pHandle, 
					"DSPNode_GetMessage failed. Status = 0x%x\n", status);
				return FAIL;
			}
		}
	} // End of For loop - iNumTransfers


    status = DSPNode_Terminate(hNode, &exitStatus);
    status = DSPNode_Delete(hNode);

    OsSpecificMemFree(arrBuf);
	/* Return from this Thread */
	return 0;
}



/*
 *  ======== main ========
 *
 */
INT multiThreadMsgQueue_STRESS(INT argc, CHAR * argv[])
{
    	BOOL           			bUseLog=FALSE;
    	BOOL           			bVerbose=FALSE;
    	FILE                    *pHandle = NULL;
	INT                     iNumThreads = 0;
    	INT                     iNumTransfers = 0;
    	INT                     iBuffSize = 0;
	INT                     iQueue = 0;
	CHAR           			szLogName[DEFNAMESIZE];
	INT            			iMandArgCount = NUMOFCOMPARGS;
	DSP_STATUS     			status;
	DWORD		   			threadID[MAXNUMOFTHREADS];
	HANDLE		   			thandle[MAXNUMOFTHREADS];
	MultiThreadArgs 		thrdArgs;

    	struct DSP_PROCESSORATTRIN		*pAttrIn;
	DSP_HPROCESSOR			hProcessor;
    	CHAR					*aArgv[DEFNAMESIZE] = { DSPEXEC, "" };
	INT						iArgc = 1;
	INT						opt;
	INT						index=0;
    	INT                 	retVal = PASS;
    	BOOL					fLoadDsp = FALSE;
	


	/* Check Number of Arguments */
	/* Process arguments and Store the values */
	/* Load the Image on DSP */
	/* Create requested number of Threads and pass the
	   input params to these Threads */
	/* In each of the Thread do the following: */
		/* Allocate Buffers for Transfer of Msg */
		/* Set the Properties, Allocate and Create the Node on DSP */
		/* Run the task */
		/* Q the requested num of messages */
		/* check if the msg transfer was complete */
	/* End !! */



	/* Process arguments and Store the values */
    optind = 0;
	opt = getopt(argc, argv, "+vl:x:t:n:s:q:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("MultiThreadMsgQueueTx_STRESS: Verbose Mode On\n");
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "DSPStream_Issue:");
            TST_verbose("Log opened as: %s\n", szLogName);
            // TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            fLoadDsp = TRUE;
            break;
		case 't':
			/* Number of Threads to be created */
			iNumThreads = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Number of Threads to be Created = %d\n",iNumThreads);
            TSTLOG_appendLog(pHandle,"Number of Threads to be Created = %d\n",iNumThreads);
			break;
		case 'n':
			/* Number of Transfers */
			iNumTransfers = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Number of transfers set to %d\n",iNumTransfers);
            TSTLOG_appendLog(pHandle, "Number of transfer set to %d",iNumTransfers);
			break;
		case 's':
			/* Buffer Size */
			iBuffSize = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Buffer Size set to %d\n",iBuffSize);
            TSTLOG_appendLog(pHandle, "Buffer Size set to %d\n",iBuffSize);
			break;
		case 'q':
			/* Number of Messages to be Queued */
			iQueue = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Num of msgs to be Qd %d\n",iQueue);
            TSTLOG_appendLog(pHandle, "Num of msgs to be Qd %d\n",iQueue);
			break;
        default:
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);

            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt(argc, argv, "+vl:x:t:n:s:q:");
    }

	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		/* Error */
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing - FAIL\n");
		return 0;
	}

    TST_verbose("DSP Executable: %s\n", aArgv[0]);

    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
	if (pAttrIn == NULL)
	{
		// Error
		TST_verbose("Malloc for pAttrIn FAILED !!\n");
		graceful_exit();
	}
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Attach to processor */
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    INT                     procId=0;
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, pAttrIn, &hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    }
    
    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Attach()\n");
        }
        graceful_exit();
    }

    if(fLoadDsp)
    {
	printf("BOF name: %s\n",aArgv[0]);
    /* Load default program */
    status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR**)aArgv, NULL);
    if (!DSP_SUCCEEDED(status)) 
	{
		TSTLOG_appendLog(pHandle, "Status Val: %x\n",status);
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Load()\n");
        }
		graceful_exit();
    }

    status = DSPProcessor_Start(hProcessor);
    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Start()\n");
        }
        graceful_exit();
	    }
    }

	/* Fill the thrdArgs which are to be passed on to the Thread */
	thrdArgs.hProcessor = hProcessor;
	thrdArgs.iNumTransfers = iNumTransfers;
	thrdArgs.iBuffSize = iBuffSize;
	thrdArgs.pHandle = pHandle;
	thrdArgs.bUseLog = bUseLog;
	thrdArgs.bVerbose = bVerbose;
	thrdArgs.iQueue = iQueue;

	/* Create requested number of Threads and pass the
	   input params to these Threads */
	for (index = 0; index < iNumThreads; index ++)
	{
		thandle[index] = (PVOID)OsSpecificThreadCreate(&threadFxn, &thrdArgs, &threadID[index]);
			
		if (thandle[index] == NULL)
		{
			/* ERROR: Could not create Thread ! Exit !! */
		}
	}

	for (index = 0; index < iNumThreads; index ++)
	{
		OsSpecificThreadResume(thandle[index]);
	}


	// Wait for the threads to finish before doing a Detach.
	// Code needs to be added here !!! 
	for (index = 0; index < iNumThreads; index ++)
	{
	}

    DSPProcessor_Detach(hProcessor);

    OsSpecificMemFree(pAttrIn);

    if (retVal == PASS) 
	{
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

