/*
 *  Copyright 2001-2008 Texas Instruments - http://www.ti.com/
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

 
/*
 *  ======== resource.c ========
 *  @(#) DSP/BIOS Bridge 
 *  Description:
 *     Console application to manage the DSP/BIOS Bridge Ping Task Node.
 *
 *  Demonstrates:
 *      1. How to create a task node with initial arguments.
 *      2. How to get notified of, and retrieve messages, sent from a DSP node.
 *
 *  Requires:
 *      1. Node PING_TI must be configured into the base image (via Gconf)
 *
 *  Usage:
 *      resource.out <message_count>
 *
 *  Notes:
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <dbapi.h>                  /* DSP/BIOS Bridge APIs                  */
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>

#define DEFAULTDELAY  "50"          /* default resource delay               */
#define DEFAULTMSGS   50            /* default message count                 */
#define DSPUNIT       0             /* Default (first) DSP ID                */
#define ARGSIZE       32            /* Size of arguments to Ping node.       */
#define MAXNAMELEN    64            /* Max length of event name.             */
#define MAXMSGLEN     128           /* Max length of MessageBox msg.         */


#define PING           0x01

/* PING_TI_UUID = 12A3C3C1_D015_11D4_9F69_00C04F3A59AE */
static const struct DSP_UUID PING_TI_uuid = {
    0x12a3c3c1, 0xd015, 0x11d4, 0x9f, 0x69, {
    0x00, 0xc0, 0x4f, 0x3a, 0x59, 0xae
    }
};

/* Ping task context data structure - passed between functions in this module */
struct PING_TASK {
    DSP_HPROCESSOR  hProcessor;     /* Handle to processor.             */
    DSP_HNODE       hNode;          /* Handle to node.                  */
    HANDLE          hEvent;         /* Event to be signalled by DSP     */
    UINT            msgCount;       /* Number of messages DSP sends     */
    UINT            choice;
} ;

/* Argument structure for DSPNode_Allocate():                           */
struct PING_NODEDATA {
    ULONG   cbData;
    BYTE    cData[ARGSIZE];
} ;

/* Forward Declarations */
static DSP_STATUS ProcessArgs        (int argc, char ** argv, UINT * pMsgCnt,
                                      struct PING_NODEDATA *argsBuf);
static DSP_STATUS AttachProcessor    (struct PING_TASK *resourceTask);
static DSP_STATUS CreateNode         (struct PING_TASK *resourceTask,
                                      struct PING_NODEDATA *argsBuf);
static DSP_STATUS RunNode            (struct PING_TASK *resourceTask);
static DSP_STATUS DestroyNode        (struct PING_TASK *resourceTask);
static DSP_STATUS DetachProcessor    (struct PING_TASK *resourceTask);


/*
 *  ======== main ========
 */
int main(int argc, char ** argv)
{
    struct PING_TASK     resourceTask;       /* Ping task context */
    UINT          msgCount;           /* Number of messages DSP sends         */
    UINT          choice;           /* Number of messages DSP sends         */
    struct PING_NODEDATA argsBuf;            /* Task Node args. */
    DSP_STATUS    status = DSP_SOK;

#ifndef RES_CLEANUP_DISABLE
    /* Process command line arguments, open data files: */
    status = ProcessArgs(argc, argv, &msgCount, &argsBuf);
    if (DSP_SUCCEEDED(status)) {
      sscanf(argv[1], "%u", &(choice));
      resourceTask.choice = choice;
        /* Initialize context: */
        resourceTask.hProcessor = NULL;
        resourceTask.hNode = NULL;
        resourceTask.hEvent = NULL;
        resourceTask.msgCount = msgCount ;

        status = DspManager_Open (0, NULL);

        if (DSP_SUCCEEDED(status)) {

            /* Perform processor level initialization. */
            status = AttachProcessor(&resourceTask);
            if (DSP_SUCCEEDED(status)) {

              /* Perform node level initialization. */
              status = CreateNode(&resourceTask, &argsBuf);
              if (DSP_SUCCEEDED(status)) {
                  /* Run Ping task. */
                 status = RunNode(&resourceTask);
              }
            }

            /* Cleanup all allocated resources. */
            DestroyNode(&resourceTask);
            DetachProcessor(&resourceTask);
        }
    }
    status = DspManager_Close(0,NULL);
#else
printf("Warning: Resource Cleanup is not enabled in Bridge, Build bridge "
									"without RES_CLEANUP_DISABLE Flag !!\n");
#endif
    return (DSP_SUCCEEDED(status) ? 0 : -1);
}


#ifndef RES_CLEANUP_DISABLE
/*
 *  ======== AttachProcessor ========
 *  Perform processor related initialization.
 */
static DSP_STATUS AttachProcessor(struct PING_TASK *resourceTask)
{
    DSP_STATUS          status = DSP_EFAIL;
    struct DSP_PROCESSORINFO   dspInfo;
    UINT                numProcs;
    UINT                index=0;
    INT                 procId = 0;

    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index, &dspInfo,
						(UINT) sizeof(struct DSP_PROCESSORINFO), &numProcs))) {
        if ((dspInfo.uProcessorType == DSPTYPE_55) || 
									(dspInfo.uProcessorType == DSPTYPE_64)) {
            printf("DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }

    /* Attach to a DSP (first by default) */
    status = DSPProcessor_Attach(procId, NULL, &resourceTask->hProcessor);
    if (DSP_FAILED(status)) {
       fprintf(stdout, "DSPProcessor_Attach failed: 0x%x\n", (UINT)status);
    }

    return (status);
}


/*
 *  ======== CreateNode ========
 *  Perform node related initialization.
 */
static DSP_STATUS CreateNode(struct PING_TASK *resourceTask, 
												struct PING_NODEDATA *argsBuf)
{
    struct DSP_CBDATA          *pArgs;
    DSP_STATUS          status;

    pArgs = (struct DSP_CBDATA *)argsBuf;

    /* Allocate the resource node, passing arguments for node create phase. */
    status = DSPNode_Allocate(resourceTask->hProcessor, &PING_TI_uuid, pArgs,
           NULL, &resourceTask->hNode);
    if (DSP_FAILED(status)) {
       fprintf(stdout, "DSPNode_Allocate failed: 0x%x\n", (UINT)status);
    }

    /* Create the resource node on the DSP. */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPNode_Create(resourceTask->hNode);
        if (DSP_FAILED(status)) {
            fprintf(stdout, "DSPNode_Create failed: 0x%x\n", (UINT)status);
        }
    }

    /* Register for message notifications:  */
    if (DSP_SUCCEEDED(status)) {

        resourceTask->hEvent = malloc (sizeof(struct DSP_NOTIFICATION));

        status = DSPNode_RegisterNotify(resourceTask->hNode,
                                    DSP_NODEMESSAGEREADY, DSP_SIGNALEVENT,
														resourceTask->hEvent);
        if (DSP_FAILED(status)) {
            fprintf(stdout, "DSPNode_RegisterNotify failed: 0x%x\n",
																(UINT)status);
        }
    }

    return (status);
}


/*
 *  ======== RunNode ========
 *  Run resource task.
 */
static DSP_STATUS RunNode(struct PING_TASK *resourceTask)
{
    DSP_STATUS          status;
    DSP_STATUS          exitStatus;
    struct DSP_MSG             dspMsg;
    UINT                uIndex;
    DWORD               cEvents;
    struct DSP_NOTIFICATION*   ahEvents[2];
    int                 n = 0;
    BYTE                *pBuf = NULL;
    cEvents     = 1;
    ahEvents[0] = resourceTask->hEvent;
    UINT length = 0, uSize = 0;
    UINT pid, parent_pid, choice, stat;
    /* Start the resource node on the DSP. */
    status = DSPNode_Run(resourceTask->hNode);
    if (DSP_FAILED(status)) {
       fprintf(stdout, "DSPNode_Run failed: 0x%x\n", (UINT)status);
    }

   switch(resourceTask->choice) {
      case 0:
        printf("********* Segmantaion fault test case *********\n");
        printf("%c",*pBuf);
      break;

      case 1:
        printf("********* Cleaning unreleased resources test case *********\n");
      if(!(pid = fork()))
      {
        fprintf(stdout, "\nAbout to call execl\n");
        stat = execl("strmcopy.out","0","resource.out","resource.out","out.txt",
																		NULL);
        fprintf(stdout, "\nexecl stat:%x\n", stat);
        perror("The exec failed!");
      }
      else
      {
         sleep(10);
         stat = kill(pid, SIGKILL);
         fprintf(stdout, "\nkilled child stramcopy process stat:%x\n", stat);
      }
      break;

      case 2:
        printf("********* Sending KILL (SIGKILL) signal to bridge user process"
													"testcase *********\n");
      switch (pid = fork())
      {
         case -1:
         /* Here pid is -1, the fork failed */
         /* Some possible reasons are that you're */
         /* out of process slots or virtual memory */
         perror("The fork failed!");
         break;

         case 0:
         /* pid of zero is the child */
         /* Here we're the child...what should we do? */
         /* ... */
         /* but after doing it, we should do something like: */
         parent_pid = getppid();
         kill(parent_pid, SIGKILL);
         _exit(0);

         default:
         /* pid greater than zero is parent getting the child's pid */
         printf("Child's pid is %d\n",pid);
      }/*switch*/
      break;

    }/*switch*/
   
#if 0 
    if (DSP_SUCCEEDED(status)) {
        pBuf = (void *)malloc(8092);
      
        printf("\nCalling result of DSPManager_GetProcResourceInfo\n");
        DSPManager_GetProcResourceInfo(pBuf,&uSize);
        printf("\nCalled result of DSPManager_GetProcResourceInfo:uSize%d\n",
																		uSize);
        for(length = 0; length < uSize; length++)
        {
           printf("%c",*pBuf);
           pBuf++;
        }
        free(pBuf);
    }
#endif
    
    /* Terminate the DSP node. */
    status = DSPNode_Terminate(resourceTask->hNode, &exitStatus);
    if (DSP_FAILED(status)) {
        fprintf(stdout, "DSPNode_Terminate failed: 0x%x\n", (UINT)status);
    }

    return (status);
}


/*
 *  ======== DestroyNode ========
 *  Perform node related cleanup.
 */
static DSP_STATUS DestroyNode(struct PING_TASK *resourceTask)
{
    DSP_STATUS      status = DSP_SOK;

    if (resourceTask->hNode) {
        /* Delete DSP node. */
        status = DSPNode_Delete(resourceTask->hNode);
        if (DSP_FAILED(status)) {
            fprintf(stdout, "DSPNode_Delete failed: 0x%x\n", (UINT)status);
        }

        resourceTask->hNode = NULL;
    }

    if (resourceTask->hEvent) {
        free (resourceTask->hEvent);
    }

    return (status);
}


/*
 *  ======== DetachProcessor ========
 *  Perform processor related cleanup.
 */
static DSP_STATUS DetachProcessor(struct PING_TASK *resourceTask)
{
    DSP_STATUS      status = DSP_SOK;

    if (resourceTask->hProcessor) {
        /* Detach from processor. */
        status = DSPProcessor_Detach(resourceTask->hProcessor);
        if (DSP_FAILED(status)) {
            fprintf(stdout, "DSPProcessor_Detach failed: 0x%x\n", (UINT)status);
        }

        resourceTask->hProcessor = NULL;
    }

    return (status);
}


/*
 *  ======== ProcessArgs ========
 *  Process command line arguments for this sample, returning input and
 *  output file handles.
 */
static DSP_STATUS ProcessArgs(int argc, char **argv, UINT *pMsgCnt,
					                              struct PING_NODEDATA *argsBuf)
{
    DSP_STATUS  status = DSP_SOK;

        fprintf(stdout,"\nvalue of argc=%d\n",argc);
    switch (argc) {
    /*case 1:
        *pMsgCnt = DEFAULTMSGS;
        strncpy(argsBuf->cData, DEFAULTDELAY, ARGSIZE);
        break;*/

    case 2:
        sscanf(argv[1], "%u", pMsgCnt);
        strncpy(argsBuf->cData, DEFAULTDELAY, ARGSIZE);
        break;

    default:
        fprintf(stdout,
            "Usage: %s <Choice> \n", argv[0]);
        fprintf(stdout,
            "Where Choice 1: Segmentation fault\n");
        fprintf(stdout, "Where Choice 1: Terminating a process "
											"without releasing resources\n");
        fprintf(stdout,
            "Where Choice 1: Killing a process by send KILL signal\n");
        strncpy(argsBuf->cData, DEFAULTDELAY, ARGSIZE);
        status = DSP_EFAIL;
        break;
    }

    if (DSP_SUCCEEDED(status)) {
        argsBuf->cbData = strlen(argsBuf->cData) + 1;
    }
    return (status);
}

#endif

